from django.views.generic import TemplateView
from django.db.models import Count
from django.utils import timezone

from datetime import datetime
import six
from braces.views import PermissionRequiredMixin

from danceschool.core.models import Instructor, SeriesTeacher, Customer
from danceschool.core.utils.timezone import ensure_timezone

if six.PY3:
    # Ensures that checks for Unicode data types (and unicode type assignments) do not break.
    unicode = str


class SchoolStatsView(PermissionRequiredMixin, TemplateView):
    template_name = 'stats/schoolstats.html'
    permission_required = 'core.view_school_stats'

    def get_context_data(self, **kwargs):
        from .stats import getGeneralStats

        request = self.request
        context_data = kwargs

        (totalStudents,numSeries,totalSeriesRegs,totalTime) = getGeneralStats(request)

        context_data.update({
            'totalStudents':totalStudents,
            'numSeries':numSeries,
            'totalSeriesRegs':totalSeriesRegs,
            'totalTime':totalTime,
        })
        return context_data



from django.core.management.base import BaseCommand
from django.apps import apps
from django.conf import settings

from six.moves import input

try:
    import readline
except ImportError:
    pass


class Command(BaseCommand):
    help = 'Check Paypal settings and created necessary placeholders for Paypal Express Checkout integration.'

    def boolean_input(self, question, default=None):
        '''
        Method for yes/no boolean inputs
        '''
        result = input("%s: " % question)
        if not result and default is not None:
            return default
        while len(result) < 1 or result[0].lower() not in "yn":
            result = input("Please answer yes or no: ")
        return result[0].lower() == "y"

    def handle(self, *args, **options):

        from cms.api import add_plugin
        from cms.models import Page, StaticPlaceholder

        try:
            initial_language = settings.LANGUAGES[0][0]
        except IndexError:
            initial_language = getattr(settings, 'LANGUAGE_CODE', 'en')

        # Do some sanity checks to ensure that necessary apps are listed in INSTALLED_APPS
        # before proceeding
        required_apps = [
            ('cms', 'Django CMS'),
            ('danceschool.core', 'Core danceschool app'),
            ('danceschool.payments.paypal', 'Paypal integration app'),
            ('dynamic_preferences', 'django-dynamic-preferences'),
        ]
        for this_app in required_apps:
            if not apps.is_installed(this_app[0]):
                self.stdout.write(self.style.ERROR('ERROR: %s is not installed or listed in INSTALLED_APPS. Please install before proceeding.' % this_app[1]))
                return None

        self.stdout.write(
            """
CHECKING PAYPAL INTEGRATION
---------------------------
            """
        )

        mode = getattr(settings,'PAYPAL_MODE',None)
        client_id = getattr(settings,'PAYPAL_CLIENT_ID','')
        client_secret = getattr(settings,'PAYPAL_CLIENT_SECRET','')

        if mode in ['sandbox','live']:
            self.stdout.write('Paypal Mode: %s, OK' % mode)
        else:
            self.stdout.write(self.style.WARNING('Paypal Mode: not set'))

        if client_id:
            self.stdout.write('Paypal client ID set.')
        else:
            self.stdout.write(self.style.WARNING('Paypal client ID not set'))

        if client_secret:
            self.stdout.write('Paypal client secret set.')
        else:
            self.stdout.write(self.style.WARNING('Paypal client secret not set.'))

        if mode in ['sandbox','live'] and client_id and client_secret:
            try:
                import paypalrestsdk
                paypalrestsdk.configure({
                    'mode': mode or 'sandbox',
                    'client_id': client_id,
                    'client_secret': client_secret,
                })
                paypalrestsdk.Payment.all({'count': 1})
            except ImportError:
                self.stdout.write(self.style.ERROR('Required paypalrestsdk app not installed.'))
            except paypalrestsdk.exceptions.UnauthorizedAccess:
                self.stdout.write(self.style.ERROR('Unauthorized credentials supplied for Paypal API.'))
            else:
                self.stdout.write(self.style.SUCCESS('Successfully connected to Paypal using API credentials.'))

        add_paypal_paynow = self.boolean_input('Add Paypal Pay Now link to the registration summary view to allow students to pay [Y/n]', True)
        if add_paypal_paynow:
            home_page = Page.objects.filter(is_home=True,publisher_public=True).first()
            if not home_page:
                self.stdout.write(self.style.ERROR('Cannot add Pay Now link because a home page has not yet been set.'))
            else:
                paynow_sp = StaticPlaceholder.objects.get_or_create(code='registration_payment_placeholder')
                paynow_p_draft = paynow_sp[0].draft
                paynow_p_public = paynow_sp[0].public

                if paynow_p_public.get_plugins().filter(plugin_type='CartPaymentFormPlugin').exists():
                    self.stdout.write('Paypal Pay Now button already present.')
                else:
                    add_plugin(
                        paynow_p_draft, 'CartPaymentFormPlugin', initial_language,
                        successPage=home_page,
                    )
                    add_plugin(
                        paynow_p_public, 'CartPaymentFormPlugin', initial_language,
                        successPage=home_page,
                    )
                    self.stdout.write('Paypal Pay Now link added.')



from django.db import models
from django.conf import settings
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from cms.models.pluginmodel import CMSPlugin
from cms.models.fields import PageField

import logging

from danceschool.core.models import PaymentRecord


# Define logger for this file
logger = logging.getLogger(__name__)


@python_2_unicode_compatible
class StripeCharge(PaymentRecord):

    def __init__(self, *args, **kwargs):
        super(StripeCharge, self).__init__(*args, **kwargs)

        # bring in stripe, and get the api key from settings.py
        import stripe
        stripe.api_key = getattr(settings,'STRIPE_PRIVATE_KEY','')

        self.stripe = stripe

    # store the stripe charge id for this sale
    chargeId = models.CharField(_('Charge Id'),max_length=32)
    status = models.CharField(_('Current status'),max_length=30,null=True,blank=True)

    @property
    def methodName(self):
        return 'Stripe Charge'

    @property
    def recordId(self):
        return self.chargeId

    @property
    def netAmountPaid(self):
        charge = self.getCharge()
        return float(charge.amount - charge.amount_refunded) / 100

    @property
    def refundable(self):
        return True

    def getCharge(self):
        return self.stripe.Charge.retrieve(self.chargeId)

    def getPayerEmail(self):
        charge = self.getCharge()
        customer = self.stripe.Customer.retrieve(charge.customer)
        return customer.email

    def refund(self, amount=None):
        refundData = []

        refund_kwargs = {
            'charge': self.chargeId,
            'reason': 'requested_by_customer',
        }
        if amount:
            refund_kwargs['amount'] = int(amount * 100)

        refund = self.stripe.Refund.create(**refund_kwargs)

        if refund.status == 'succeeded':
            bt = self.stripe.BalanceTransaction.retrieve(refund.balance_transaction)
            refundData.append({
                'status': 'success',
                'refund_id': refund.id,
                'refundAmount': float(refund.amount) / 100,
                'fees': float(bt.fee) / 100,
            })
        else:
            logger.error('Error processing refund.')
            refundData.append({'status': 'error', 'errors': refund.status})

        return refundData

    class Meta:
        verbose_name = _('Stripe charge')
        verbose_name_plural = _('Stripe charges')


class StripeChargeFormModel(CMSPlugin):
    ''' This model holds options for instances of the GiftCertificateFormPlugin and the CartPaymentFormPlugin '''

    successPage = PageField(verbose_name=_('Success Page'),help_text=_('When the user returns to the site after a successful transaction, send them to this page.'),related_name='successPageForStripe')
    defaultAmount = models.FloatField(verbose_name=_('Default amount'),help_text=_('The initial value for gift certificate forms.'),default=0)

    def get_short_description(self):
        return self.plugin_type or self.id



from dynamic_preferences.serializers import BaseSerializer, UNSET


class PageModelSerializer(BaseSerializer):
    '''
    The Page selector field needs its own serializer, which accepts a Page object,
    but returns an int.  So, when accessing the value of the DefaultAdminSuccessPage
    constant, one must use Page.objects.get(pk=getConstant('general__defaultAdminSuccessPage')).
     '''

    @classmethod
    def to_python(cls, value, **kwargs):
        if not value or (value == UNSET):
            return

        try:
            return int(value)
        except ValueError:
            raise cls.exception("Value {0} cannot be converted to int")

    @classmethod
    def to_db(cls, value, **kwargs):
        if not value or (value == UNSET):
            return None
        return str(value.pk)



from django_ical.views import ICalFeed
from django.http import JsonResponse
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone
from django.conf import settings
from django.contrib.auth.models import User

import pytz
from datetime import datetime, timedelta

from danceschool.core.models import StaffMember
from danceschool.core.constants import getConstant
from danceschool.core.utils.timezone import ensure_timezone

from .models import EventOccurrence


# Because our calendar will have both series classes and non-recurring events
# in it, we need to create a custom class with properties assigned appropriately
# to iterate through in the feed generation process
class EventFeedItem(object):

    def __init__(self,object,**kwargs):
        timeZone = pytz.timezone(getattr(settings,'TIME_ZONE','UTC'))
        if kwargs.get('timeZone',None):
            try:
                timeZone = pytz.timezone(kwargs.get('timeZone',None))
            except pytz.exceptions.UnknownTimeZoneError:
                pass

        self.id = 'privateEvent_' + str(object.event.id)
        self.type = 'privateEvent'
        self.id_number = object.event.id
        self.title = object.event.name
        self.description = object.event.description

        if hasattr(object.event,'category') and object.event.category:
            self.category = object.event.category.name
            self.color = object.event.category.displayColor
        else:
            self.category = None
            self.color = getConstant('calendar__defaultEventColor')
        self.start = timezone.localtime(object.startTime,timeZone)
        self.end = timezone.localtime(object.endTime,timeZone)
        self.allDay = object.allDayForDate(object.startTime)
        if hasattr(object,'event.location'):
            self.location = object.event.location.name + '\n' + object.event.location.address + '\n' + object.event.location.city + ', ' + object.event.location.state + ' ' + object.event.location.zip
            self.room = getattr(object.event,'room',None)
        else:
            self.location = None
            self.room = None
        self.url = object.event.link


class EventFeed(ICalFeed):
    """
    A simple event calender
    """
    timezone = getattr(settings,'TIME_ZONE','UTC')
    description = _('Calendar for %s' % getConstant('contact__businessName'))

    def get_member(self,obj):
        member = None
        try:
            member = StaffMember.objects.get(userAccount=obj)
        except ObjectDoesNotExist:
            pass
        return member

    def get_object(self,request,instructorFeedKey=''):
        if instructorFeedKey:
            try:
                return User.objects.get(staffmember__feedKey=instructorFeedKey)
            except (ValueError, ObjectDoesNotExist):
                pass
        return request.user

    def title(self,obj):
        this_instructor = self.get_member(obj)
        if not this_instructor:
            return _('%s Events Calendar' % getConstant('contact__businessName'))
        return _('%s Staff Calendar for %s' % (getConstant('contact__businessName'), this_instructor.fullName))

    def items(self,obj):
        if not getattr(obj,'is_staff') or not getConstant('calendar__privateCalendarFeedEnabled'):
            return []

        this_user = obj
        instructor_groups = list(this_user.groups.all().values_list('id',flat=True))

        occurrences = EventOccurrence.objects.filter(event__privateevent__isnull=False).filter(
            Q(event__privateevent__displayToGroup__in=instructor_groups) |
            Q(event__privateevent__displayToUsers=this_user) |
            (Q(event__privateevent__displayToGroup__isnull=True) & Q(event__privateevent__displayToUsers__isnull=True))).order_by('-startTime')

        return [EventFeedItem(x) for x in occurrences]

    def item_guid(self, item):
        return item.id + '@' + item.url

    def item_title(self, item):
        return item.title

    def item_link(self, item):
        return item.url

    def item_location(self, item):
        return item.location

    def item_description(self, item):
        return item.description

    def item_start_datetime(self, item):
        return item.start

    def item_end_datetime(self, item):
        return item.end


def json_event_feed(request,location_id=None,room_id=None):
    '''
    The Jquery fullcalendar app requires a JSON news feed, so this function
    creates the feed from upcoming PrivateEvent objects
    '''

    if not getConstant('calendar__privateCalendarFeedEnabled') or not request.user.is_staff:
        return JsonResponse({})
    this_user = request.user

    startDate = request.GET.get('start','')
    endDate = request.GET.get('end','')
    timeZone = request.GET.get('timezone',getattr(settings,'TIME_ZONE','UTC'))

    time_filter_dict_events = {}
    if startDate:
        time_filter_dict_events['startTime__gte'] = ensure_timezone(datetime.strptime(startDate,'%Y-%m-%d'))
    if endDate:
        time_filter_dict_events['endTime__lte'] = ensure_timezone(datetime.strptime(endDate,'%Y-%m-%d')) + timedelta(days=1)

    instructor_groups = list(this_user.groups.all().values_list('id',flat=True))

    filters = Q(event__privateevent__isnull=False) & (
        Q(event__privateevent__displayToGroup__in=instructor_groups) |
        Q(event__privateevent__displayToUsers=this_user) |
        (Q(event__privateevent__displayToGroup__isnull=True) & Q(event__privateevent__displayToUsers__isnull=True))
    )

    if location_id:
        filters = filters & Q(event__location__id=location_id)
    if room_id:
        filters = filters & Q(event__room_id=room_id)

    occurrences = EventOccurrence.objects.filter(filters).filter(**time_filter_dict_events).order_by('-startTime')

    eventlist = [EventFeedItem(x,timeZone=timeZone).__dict__ for x in occurrences]

    return JsonResponse(eventlist,safe=False)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-20 16:43
from __future__ import unicode_literals

from django.db import migrations, models
import djangocms_text_ckeditor.fields


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0009_auto_20170719_1720'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailtemplate',
            name='content',
            field=models.TextField(blank=True, help_text='See the list of available variables for details on what information can be included with template tags.', null=True, verbose_name='Plain text Content'),
        ),
        migrations.AlterField(
            model_name='emailtemplate',
            name='html_content',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, help_text='Emails are sent as plain text by default.  To send an HTML email instead, enter your desired content in this field.', null=True, verbose_name='HTML rich text content'),
        ),
    ]



from django.dispatch import receiver
from django.db.models import Q
from django.db.models.signals import post_save, m2m_changed
from django.utils.translation import ugettext_lazy as _

import sys
import logging

from danceschool.core.models import EventStaffMember, EventOccurrence, InvoiceItem, Invoice
from danceschool.core.constants import getConstant

from .models import ExpenseItem, RevenueItem, RepeatedExpenseRule


# Define logger for this file
logger = logging.getLogger(__name__)


@receiver(m2m_changed, sender=EventStaffMember.occurrences.through)
def modifyExistingExpenseItemsForEventStaff(sender,instance,**kwargs):
    if 'loaddata' in sys.argv or ('raw' in kwargs and kwargs['raw']):
        return
    if kwargs.get('action',None) != 'post_add':
        return

    logger.debug('ExpenseItem signal fired for EventStaffMember %s.' % instance.pk)

    staff_expenses = ExpenseItem.objects.filter(
        event=instance.event,
        expenseRule__in=instance.staffMember.expenserules.all(),
        expenseRule__applyRateRule=RepeatedExpenseRule.RateRuleChoices.hourly,
    )

    if staff_expenses:
        logger.debug('Updating existing expense items for event staff member.')
        # Fill in the updated hours and the updated total.  Set the expense item
        # to unapproved.
        for expense in staff_expenses:
            logger.debug('Updating expense item %s.' % expense.id)
            expense.hours = instance.netHours
            expense.total = expense.hours * expense.wageRate
            expense.approved = False
            expense.save()

    if hasattr(instance.replacedStaffMember,'staffMember'):
        logger.debug('Adjusting totals for replaced event staff member.')
        replaced_expenses = ExpenseItem.objects.filter(
            event=instance.event,
            expenseRule__staffmemberwageinfo__staffMember=instance.replacedStaffMember.staffMember,
            expenseRule__applyRateRule=RepeatedExpenseRule.RateRuleChoices.hourly,
        )

        # Fill in the updated hours and the updated total.  Set the expense item
        # to unapproved.
        for expense in replaced_expenses:
            logger.debug('Updating expense item %s' % expense.id)
            expense.hours = instance.replacedStaffMember.netHours
            expense.total = expense.hours * expense.wageRate
            expense.approved = False
            expense.save()


@receiver(post_save, sender=EventOccurrence)
def modifyExistingExpenseItemsForSeriesClass(sender,instance,**kwargs):
    if 'loaddata' in sys.argv or ('raw' in kwargs and kwargs['raw']):
        return

    logger.debug('ExpenseItem signal fired for EventOccurrence %s.' % instance.id)

    staff_expenses = ExpenseItem.objects.filter(
        event=instance.event,
        expenseRule__staffmemberwageinfo__isnull=False,
        expenseRule__applyRateRule=RepeatedExpenseRule.RateRuleChoices.hourly,
    )

    # Fill in the updated hours and the updated total.  Set the expense item
    # to unapproved.
    for expense in staff_expenses:
        esm_filters = Q(event=expense.event) & Q(staffMember=expense.expenseRule.staffMember)
        if expense.expenseRule.category:
            esm_filters = esm_filters & Q(category=expense.expenseRule.category)

        # In instances where the expense rule does not specify a category, there could
        # be more than one EventStaffMember object for a given staffMember at the
        # same Event.  There is no easy way to identify which expense is which in this instance,
        # so when EventOccurrences are modified, these expenses will not update.
        eventstaffmembers = EventStaffMember.objects.filter(esm_filters)
        if eventstaffmembers.count() == 1:
            esm = eventstaffmembers.first()
            expense.hours = esm.netHours
            expense.total = expense.hours * expense.wageRate
            expense.approved = False
            expense.save()


@receiver(post_save, sender=InvoiceItem)
def createRevenueItemForInvoiceItem(sender,instance,**kwargs):
    if 'loaddata' in sys.argv or ('raw' in kwargs and kwargs['raw']):
        return

    logger.debug('RevenueItem signal fired for InvoiceItem %s.' % instance.id)

    received_status = (instance.invoice.status == Invoice.PaymentStatus.paid)

    related_item = getattr(instance, 'revenueitem', None)
    if not related_item:
        related_item = RevenueItem.objects.create(
            invoiceItem=instance,
            invoiceNumber=instance.id,
            grossTotal=instance.grossTotal,
            total=instance.total,
            adjustments=instance.adjustments,
            fees=instance.fees,
            taxes=instance.taxes,
            buyerPaysSalesTax=instance.invoice.buyerPaysSalesTax,
            category=getConstant('financial__registrationsRevenueCat'),
            submissionUser=instance.invoice.submissionUser,
            currentlyHeldBy=instance.invoice.collectedByUser,
            received=received_status,
            paymentMethod=instance.invoice.get_payment_method(),
            description=_('Registration invoice %s' % instance.id)
        )
        logger.debug('RevenueItem created.')
    else:
        # Check that the existing revenueItem is still correct
        saveFlag = False

        for field in ['grossTotal','total','adjustments','fees','taxes']:
            if getattr(related_item,field) != getattr(instance,field):
                setattr(related_item,field,getattr(instance,field))
                saveFlag = True
        for field in ['buyerPaysSalesTax',]:
            if getattr(related_item,field) != getattr(instance.invoice,field):
                setattr(related_item,field,getattr(instance.invoice,field))
                saveFlag = True

        if related_item.received != received_status:
            related_item.received = received_status
            related_item.paymentMethod = instance.invoice.get_payment_method()
            saveFlag = True

        if saveFlag:
            related_item.save()
            logger.info('RevenueItem associated with InvoiceItem %s updated.' % instance.id)



from django import forms
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.core.exceptions import ValidationError

from datetime import datetime, timedelta
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Div, Submit

from danceschool.core.constants import getConstant
from danceschool.core.models import DanceRole, Location, Room, Instructor, PricingTier
from danceschool.core.utils.timezone import ensure_localtime
from danceschool.core.forms import LocationWithDataWidget

from .models import InstructorAvailabilitySlot


def get_duration_choices():
    return [(x,x) for x in range(
        getConstant('privateLessons__minimumLessonLength'),
        getConstant('privateLessons__maximumLessonLength') + 1,
        getConstant('privateLessons__lessonLengthInterval'),
    )]


def get_default_duration():
    return getConstant('privateLessons__defaultLessonLength')


class SlotBookingForm(forms.Form):
    slotId = forms.IntegerField(required=True,widget=forms.HiddenInput)
    duration = forms.ChoiceField(label=_('Duration'),choices=get_duration_choices,initial=get_default_duration)
    role = forms.ChoiceField(label=_('Dance role'),choices=[(x.id, x.name) for x in DanceRole.objects.all()])
    participants = forms.IntegerField(label=_('Expected # Participants'),initial=1, min_value=1, help_text=_('Be advised that group lessons may be charged a different rate.'))
    comments = forms.CharField(label=_('Comments/Notes'), required=False, help_text=_('Please enter any comments or notes that you would like to be provided to the instructor before the lesson, such as the topics on which you may want to focus.'))

    def __init__(self, *args, **kwargs):
        user = kwargs.pop('user',None)

        # Initialize the default form
        super(SlotBookingForm, self).__init__(*args, **kwargs)

        # Allow users with appropriate permissions to process door registrations.
        if user and user.has_perm('core.accept_door_payments'):
            self.fields['payAtDoor'] = forms.BooleanField(required=False,label=_('Door/Invoice Registration'))


class SlotCreationForm(forms.Form):
    instructorId = forms.ModelChoiceField(label=_('Instructor'),queryset=Instructor.objects.all(),widget=forms.HiddenInput,required=True)

    startDate = forms.DateField(label=_('Start date'),required=True,widget=forms.HiddenInput)
    endDate = forms.DateField(label=_('End date'),required=True,widget=forms.HiddenInput)
    startTime = forms.TimeField(
        label=_('Start time'),required=True,
        input_formats=getattr(settings,'TIME_INPUT_FORMATS',[]) + ['%I:%M %p','%-I:%M %p','%I:%M%p','%-I:%M%p']
    )
    endTime = forms.TimeField(
        label=_('End time'),required=True,
        input_formats=getattr(settings,'TIME_INPUT_FORMATS',[]) + ['%I:%M %p','%-I:%M %p','%I:%M%p','%-I:%M%p'],
    )

    location = forms.ModelChoiceField(label=_('Location'),queryset=Location.objects.exclude(status=Location.StatusChoices.former),required=False,widget=LocationWithDataWidget)
    room = forms.ModelChoiceField(label=_('Room'),queryset=Room.objects.exclude(location__status=Location.StatusChoices.former),required=False)
    pricingTier = forms.ModelChoiceField(
        label=_('Pricing Tier'),queryset=PricingTier.objects.filter(expired=False),required=False,
        help_text=_('A pricing tier is required for online registration and payment. If your school handles scheduling, but not payment for lessons, then leave this blank.')
    )
    status = forms.ChoiceField(label=_('Initial Status'),required=True,choices=InstructorAvailabilitySlot.SlotStatus.choices, initial=InstructorAvailabilitySlot.SlotStatus.available)

    def clean(self):
        '''
        Only allow submission if there are not already slots in the submitted window,
        and only allow rooms associated with the chosen location.
        '''

        super(SlotCreationForm,self).clean()

        startDate = self.cleaned_data.get('startDate')
        endDate = self.cleaned_data.get('endDate')
        startTime = self.cleaned_data.get('startTime')
        endTime = self.cleaned_data.get('endTime')
        instructor = self.cleaned_data.get('instructorId')

        existingSlots = InstructorAvailabilitySlot.objects.filter(
            instructor=instructor,
            startTime__gt=(
                ensure_localtime(datetime.combine(startDate,startTime)) -
                timedelta(minutes=getConstant('privateLessons__lessonLengthInterval'))
            ),
            startTime__lt=ensure_localtime(datetime.combine(endDate,endTime)),
        )

        if existingSlots.exists():
            raise ValidationError(_('Newly created slots cannot overlap existing slots for this instructor.'),code='invalid')


class SlotUpdateForm(forms.Form):
    slotIds = forms.ModelMultipleChoiceField(required=True,widget=forms.MultipleHiddenInput,queryset=InstructorAvailabilitySlot.objects.all())

    updateStatus = forms.ChoiceField(label=_('Update Status'),required=True,choices=InstructorAvailabilitySlot.SlotStatus.choices, initial=InstructorAvailabilitySlot.SlotStatus.available)
    updateLocation = forms.ModelChoiceField(label=_('Update Location'),queryset=Location.objects.exclude(status=Location.StatusChoices.former),required=False,widget=LocationWithDataWidget)
    updateRoom = forms.ModelChoiceField(label=_('Room'),queryset=Room.objects.exclude(location__status=Location.StatusChoices.former),required=False)
    updatePricing = forms.ModelChoiceField(
        label=_('Update pricing'),queryset=PricingTier.objects.filter(expired=False),required=False,
        help_text=_('A pricing tier is required for online registration and payment. If your school handles scheduling, but not payment for lessons, then leave this blank.')
    )
    deleteSlot = forms.BooleanField(label=_('Delete slot'),initial=False,help_text=_('Note that only slots with no current bookings may be deleted at this time.'),required=False)


class PrivateLessonStudentInfoForm(forms.Form):
    '''
    This is the form customers use to fill out their contact info
    for private lessons that don't involve online payment only.
    '''

    firstName = forms.CharField(label=_('First Name'))
    lastName = forms.CharField(label=_('Last Name'))
    email = forms.EmailField()
    phone = forms.CharField(required=False,label=_('Telephone (optional)'),help_text=_('We may use this to notify you in event of a cancellation.'))
    agreeToPolicies = forms.BooleanField(required=True,label=_('<strong>I agree to all policies (required)</strong>'),help_text=_('By checking, you agree to abide by all policies.'))

    def __init__(self,*args,**kwargs):
        self._request = kwargs.pop('request',None)
        user = getattr(self._request,'user',None)
        payAtDoor = kwargs.pop('payAtDoor',False)

        super(PrivateLessonStudentInfoForm,self).__init__(*args,**kwargs)

        self.helper = FormHelper()
        self.helper.form_method = 'post'
        self.helper.form_tag = False  # Our template must explicitly include the <form tag>

        if user and hasattr(user,'customer') and user.customer and not payAtDoor:
            # Input existing info for users who are logged in and have signed up before
            self.fields['firstName'].initial = user.customer.first_name or user.first_name
            self.fields['lastName'].initial = user.customer.last_name or user.last_name
            self.fields['email'].initial = user.customer.email or user.email
            self.fields['phone'].initial = user.customer.phone

        self.helper.layout = Layout(
            Div('firstName','lastName','email',css_class='form-inline'),
            Div('phone',css_class='form-inline'),
            Div('agreeToPolicies',css_class='card card-body bg-light'),
            Submit('submit',_('Complete Registration'))
        )



from django.utils.translation import ugettext_lazy as _
from django.utils import timezone

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from dateutil.relativedelta import relativedelta

from danceschool.core.models import Series
from danceschool.core.mixins import PluginTemplateMixin
from danceschool.core.registries import plugin_templates_registry, PluginTemplateBase

from .models import StatsGraphPluginModel


class StatsGraphPlugin(PluginTemplateMixin, CMSPluginBase):
    model = StatsGraphPluginModel
    name = _('School Performance Graph')
    render_template = 'stats/schoolstats_timeseriesbymonth.html'
    admin_preview = True
    cache = True
    module = _('Stats')

    def render(self, context, instance, placeholder):
        ''' Allows this plugin to use templates designed for a list of locations. '''
        context = super(StatsGraphPlugin,self).render(context,instance,placeholder)

        # Javascript makes it difficult to calculate date/time differences, so instead
        # pass the most useful ones to the template context in a dictionary.  These are used
        # to show stats over different time ranges.
        limitMonthDates = {}
        for m in range(0,25):
            limitMonthDates[m] = (timezone.now() - relativedelta(months=m)).strftime('%Y-%m-%d')

        # The same for graphs that allow one to choose different years.
        recentYears = [timezone.now().year + x for x in range(-5,1)]

        series_by_year = Series.objects.order_by('year')

        if series_by_year.count() > 0:
            first_year = series_by_year.first().year
            allYears = [x for x in range(first_year,timezone.now().year + 1)]
        else:
            allYears = []

        context.update({
            'limitMonthDates': limitMonthDates,
            'recentYears': recentYears,
            'allYears': allYears,
        })
        return context


plugin_pool.register_plugin(StatsGraphPlugin)


@plugin_templates_registry.register
class StudentByMonthTemplate(PluginTemplateBase):
    template_name = 'stats/schoolstats_timeseriesbymonth.html'
    plugin = 'StatsGraphPlugin'
    description = _('Students By Month of the Year')


@plugin_templates_registry.register
class AvgByClassTypeTemplate(PluginTemplateBase):
    template_name = 'stats/schoolstats_averagebyclasstype.html'
    plugin = 'StatsGraphPlugin'
    description = _('Performance By Class Type')


@plugin_templates_registry.register
class AvgByClassTypeMonthTemplate(PluginTemplateBase):
    template_name = 'stats/schoolstats_averagebyclasstypemonth.html'
    plugin = 'StatsGraphPlugin'
    description = _('Performance By Class Type and Month of the Year')


@plugin_templates_registry.register
class CohortRetentionTemplate(PluginTemplateBase):
    template_name = 'stats/schoolstats_cohortretention.html'
    plugin = 'StatsGraphPlugin'
    description = _('Number of Classes Taken By Starting Cohort')


@plugin_templates_registry.register
class AvgByLocationTemplate(PluginTemplateBase):
    template_name = 'stats/schoolstats_averagesbylocation.html'
    plugin = 'StatsGraphPlugin'
    description = _('Performance By Location')


@plugin_templates_registry.register
class AdvanceRegistrationTemplate(PluginTemplateBase):
    template_name = 'stats/schoolstats_advanceregistration.html'
    plugin = 'StatsGraphPlugin'
    description = _('Advance Registration Time')


@plugin_templates_registry.register
class MultiClassRegistrationTemplate(PluginTemplateBase):
    template_name = 'stats/schoolstats_multiregistrations.html'
    plugin = 'StatsGraphPlugin'
    description = _('Multi-Series Registration Stats')


@plugin_templates_registry.register
class RegistrationTypesTemplate(PluginTemplateBase):
    template_name = 'stats/schoolstats_registrationtypes.html'
    plugin = 'StatsGraphPlugin'
    description = _('Student Discounts and At-The-Door Registrations')


@plugin_templates_registry.register
class ReferralCountsTemplate(PluginTemplateBase):
    template_name = 'stats/schoolstats_referralcounts.html'
    plugin = 'StatsGraphPlugin'
    description = _('Tracked Advertising Referrals')


@plugin_templates_registry.register
class BestCustomersTemplate(PluginTemplateBase):
    template_name = 'stats/schoolstats_bestcustomers.html'
    plugin = 'StatsGraphPlugin'
    description = _('Best Customers and Most Active Teachers')



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-04-25 00:10
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('core', '0001_initial'),
        ('auth', '0008_alter_user_username_max_length'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='EventReminder',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time', models.DateTimeField()),
                ('completed', models.BooleanField(default=False, help_text='This will be set to true once the reminder has been sent.')),
            ],
            options={
                'ordering': ('time',),
            },
        ),
        migrations.CreateModel(
            name='PrivateEvent',
            fields=[
                ('event_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.Event')),
                ('title', models.CharField(help_text='Give the event a title', max_length=100)),
                ('descriptionField', models.TextField(blank=True, null=True, verbose_name='Description')),
                ('locationString', models.CharField(blank=True, help_text='If this event is not at a public event location, then enter it here.', max_length=200, null=True, verbose_name='Other Location')),
                ('link', models.URLField(blank=True, help_text='Optionally include the URL to anything that may be relevant for this event.')),
            ],
            options={
                'abstract': False,
            },
            bases=('core.event',),
        ),
        migrations.CreateModel(
            name='PrivateEventCategory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Category name will be displayed.', max_length=100, unique=True)),
                ('description', models.TextField(blank=True, help_text='Add an optional description.', null=True)),
                ('displayColor', models.CharField(default='#0000FF', help_text='Choose a color for the calendar display.', max_length=10)),
                ('requiredGroup', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='auth.Group', verbose_name='Group required to add events to this category.')),
            ],
            options={
                'verbose_name_plural': 'Private events categories',
                'verbose_name': 'Private events category',
            },
        ),
        migrations.AddField(
            model_name='privateevent',
            name='category',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='private_events.PrivateEventCategory'),
        ),
        migrations.AddField(
            model_name='privateevent',
            name='displayToGroup',
            field=models.ForeignKey(blank=True, help_text='If this is set, then only these users will see this event on their calendar.', null=True, on_delete=django.db.models.deletion.CASCADE, to='auth.Group', verbose_name='Display to group'),
        ),
        migrations.AddField(
            model_name='privateevent',
            name='displayToUsers',
            field=models.ManyToManyField(blank=True, help_text='If this is set, then only chosen users will see this event on their calendar.', to=settings.AUTH_USER_MODEL, verbose_name='Display to users'),
        ),
        migrations.AddField(
            model_name='eventreminder',
            name='event',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='core.Event'),
        ),
        migrations.AddField(
            model_name='eventreminder',
            name='eventOccurrence',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='core.EventOccurrence', verbose_name='Event Occurrence'),
        ),
        migrations.AddField(
            model_name='eventreminder',
            name='notifyList',
            field=models.ManyToManyField(to=settings.AUTH_USER_MODEL, verbose_name='Notification List'),
        ),
    ]



from django.dispatch import receiver

from allauth.account.signals import email_confirmed
from allauth.account.models import EmailAddress
import logging

from .signals import post_registration
from .models import Registration


# Define logger for this file
logger = logging.getLogger(__name__)


@receiver(email_confirmed)
def linkUserToMostRecentCustomer(sender,**kwargs):
    '''
    If a new primary email address has just been confirmed, check if the user
    associated with that email has an associated customer object yet.  If not,
    then look for the customer with that email address who most recently
    registered for something and that is not associated with another user.
    Automatically associate the User with with Customer, and if missing, fill in
    the user's name information with the Customer's name.  This way, when a new
    or existing customer creates a user account, they are seamlessly linked to
    their most recent existing registration at the time they verify their email
    address.
    '''
    email_address = kwargs.get('email_address',None)

    if not email_address or not email_address.primary or not email_address.verified:
        return

    user = email_address.user

    if not hasattr(user, 'customer'):
        last_reg = Registration.objects.filter(
            customer__email=email_address.email,
            customer__user__isnull=True,
            dateTime__isnull=False
        ).order_by('-dateTime').first()

        if last_reg:
            customer = last_reg.customer
            customer.user = user
            customer.save()

            if not user.first_name and not user.last_name:
                user.first_name = customer.first_name
                user.last_name = customer.last_name
                user.save()


@receiver(post_registration)
def linkCustomerToVerifiedUser(sender, **kwargs):
    '''
    If a Registration is processed in which the associated Customer does not yet
    have a User, then check to see if the Customer's email address has been
    verified as belonging to a specific User, and if that User has an associated
    Customer.  If such a User is found, then associated this Customer with that
    User.  This way, if a new User verifies their email account before they have
    submitted any Registrations, their Customer account is seamlessly linked when
    they do complete their first Registration.
    '''
    registration = kwargs.get('registration',None)

    if not registration or (hasattr(registration.customer,'user') and registration.customer.user):
        return

    logger.debug('Checking for User for Customer with no associated registration.')

    customer = registration.customer

    verified_email = EmailAddress.objects.filter(
        email=customer.email,
        verified=True,
        primary=True,
        user__customer__isnull=True
    )

    if verified_email:
        logger.info('Found user %s to associate with customer %s' % (verified_email.user.id, customer.id))

        customer.user = verified_email.user
        customer.save()

        if not customer.user.first_name and not customer.user.last_name:
            customer.user.first_name = customer.first_name
            customer.user.last_name = customer.last_name
            customer.user.save()



from django.core.urlresolvers import reverse
from django.utils import timezone

from datetime import timedelta

from danceschool.core.constants import REG_VALIDATION_STR, updateConstant
from danceschool.core.utils.tests import DefaultSchoolTestCase
from danceschool.core.models import TemporaryRegistration

from .models import Voucher


class VouchersTest(DefaultSchoolTestCase):

    def create_voucher(self,**kwargs):

        v = Voucher(
            voucherId=kwargs.get('voucherId','TEST_VOUCHER'),
            name=kwargs.get('name','Test Voucher'),
            originalAmount=kwargs.get('originalAmount',10),
            maxAmountPerUse=kwargs.get('maxAmountPerUse',None),
            disabled=kwargs.get('disabled',False),
            expirationDate=kwargs.get('expirationDate', None),
            forPreviousCustomersOnly=kwargs.get('forPreviousCustomersOnly', False),
            forFirstTimeCustomersOnly=kwargs.get('forFirstTimeCustomersOnly', False)
        )
        v.save()
        return v

    def register_to_check_voucher(self,voucherCode,series):
        '''
        This method makes it easy to determine whether discounts are working
        correctly for a single class registration
        '''
        s = series

        response = self.client.get(reverse('registration'))
        self.assertEqual(response.status_code,200)
        self.assertIn(s, response.context['regOpenSeries'])

        # Sign up for the series, and check that we proceed to the student information page.
        # Because of the way that roles are encoded on this form, we just grab the value to pass
        # from the form itself.
        post_data = {'event_%s' % s.id: response.context_data['form'].fields['event_%s' % s.id].choices[0][0]}

        response = self.client.post(reverse('registration'),post_data,follow=True)
        self.assertEqual(response.redirect_chain,[(reverse('getStudentInfo'), 302)])

        tr = TemporaryRegistration.objects.get(id=self.client.session[REG_VALIDATION_STR].get('temporaryRegistrationId'))
        self.assertTrue(tr.temporaryeventregistration_set.filter(event__id=s.id).exists())
        self.assertEqual(tr.payAtDoor, False)

        # Check that the student info page lists the correct item amounts and subtotal
        self.assertEqual(tr.temporaryeventregistration_set.get(event__id=s.id).price, s.getBasePrice())
        self.assertEqual(response.context_data.get('subtotal'), s.getBasePrice())

        # Continue to the summary page
        post_data = {
            'firstName': 'Voucher',
            'lastName': 'Customer',
            'email': 'test@customer.com',
            'agreeToPolicies': True,
            'gift': voucherCode,
        }
        return self.client.post(reverse('getStudentInfo'),post_data,follow=True)

    def test_nonexistent_voucher(self):
        ''' Check that entering a non-existent voucher fails '''

        updateConstant('vouchers__enableVouchers', True)
        s = self.create_series(pricingTier=self.defaultPricing)

        response = self.register_to_check_voucher('MADEUP_CODE',s)
        self.assertEqual(response.status_code,200)
        self.assertFalse(response.redirect_chain)
        self.assertTrue(response.context_data['form'].errors.get('gift'))

    def test_disabled_voucher(self):
        ''' Create a disabled voucher and ensure that it fails '''

        updateConstant('vouchers__enableVouchers', True)
        s = self.create_series(pricingTier=self.defaultPricing)
        v = self.create_voucher(disabled=True)

        response = self.register_to_check_voucher(v.voucherId,s)
        self.assertEqual(response.status_code,200)
        self.assertFalse(response.redirect_chain)
        self.assertTrue(response.context_data['form'].errors.get('gift'))

    def test_expired_voucher(self):
        '''
        Create a voucher that has an expiration date of yesterday
        and ensure that it fails
        '''

        updateConstant('vouchers__enableVouchers', True)
        s = self.create_series(pricingTier=self.defaultPricing)
        v = self.create_voucher(expirationDate=timezone.now() + timedelta(days=-1))

        response = self.register_to_check_voucher(v.voucherId,s)
        self.assertEqual(response.status_code,200)
        self.assertFalse(response.redirect_chain)
        self.assertTrue(response.context_data['form'].errors.get('gift'))

    def test_existing_customers_voucher(self):
        '''
        Create a voucher for existing customers only and ensure
        that it fails for an anonymous new customer
        '''

        updateConstant('vouchers__enableVouchers', True)
        s = self.create_series(pricingTier=self.defaultPricing)
        v = self.create_voucher(
            forPreviousCustomersOnly=True,
            expirationDate=timezone.now() + timedelta(days=1),
        )

        response = self.register_to_check_voucher(v.voucherId,s)
        self.assertEqual(response.status_code,200)
        self.assertFalse(response.redirect_chain)
        self.assertTrue(response.context_data['form'].errors.get('gift'))

    def test_vouchers_disabled(self):
        '''
        Disable vouchers and ensure that the voucher fails
        '''

        updateConstant('vouchers__enableVouchers', False)
        s = self.create_series(pricingTier=self.defaultPricing)
        v = self.create_voucher(
            expirationDate=timezone.now() + timedelta(days=1),
        )

        response = self.register_to_check_voucher(v.voucherId,s)
        self.assertEqual(response.status_code,200)
        self.assertFalse(response.redirect_chain)
        self.assertTrue(response.context_data['form'].errors.get('gift'))

    def test_maxamountperuse(self):
        '''
        Ensure that a voucher with a max amount per use succeeds, but
        only subtracts the max amount per use.
        '''

        updateConstant('vouchers__enableVouchers', True)
        s = self.create_series(pricingTier=self.defaultPricing)
        v = self.create_voucher(
            expirationDate=timezone.now() + timedelta(days=1),
            maxAmountPerUse=2,
        )

        response = self.register_to_check_voucher(v.voucherId,s)
        self.assertEqual(response.status_code,200)
        self.assertEqual(response.redirect_chain,[(reverse('showRegSummary'), 302)])
        self.assertEqual(response.context_data.get('totalPrice'), s.getBasePrice())
        self.assertEqual(response.context_data.get('netPrice'),response.context_data.get('totalPrice') - v.maxAmountPerUse)
        self.assertEqual(response.context_data.get('is_free'),False)
        self.assertEqual(response.context_data.get('total_voucher_amount'),v.maxAmountPerUse)
        self.assertIn(v.name, response.context_data.get('voucher_names'))

        tvu = v.temporaryvoucheruse_set.filter(registration=response.context_data.get('registration'))
        self.assertTrue(tvu.exists() and tvu.count() == 1)
        self.assertEqual(tvu.first().amount, v.maxAmountPerUse)

    def test_fullamountused(self):
        '''
        Remove the max amount per use restriction and ensure that the
        voucher is applied for the full $10
        '''
        updateConstant('vouchers__enableVouchers', True)
        s = self.create_series(pricingTier=self.defaultPricing)
        v = self.create_voucher()

        response = self.register_to_check_voucher(v.voucherId,s)
        self.assertEqual(response.status_code,200)
        self.assertEqual(response.redirect_chain,[(reverse('showRegSummary'), 302)])
        self.assertEqual(response.context_data.get('totalPrice'), s.getBasePrice())
        self.assertEqual(response.context_data.get('netPrice'),response.context_data.get('totalPrice') - v.originalAmount)
        self.assertEqual(response.context_data.get('is_free'),False)
        self.assertEqual(response.context_data.get('total_voucher_amount'),v.originalAmount)
        self.assertIn(v.name, response.context_data.get('voucher_names'))

        tvu = v.temporaryvoucheruse_set.filter(registration=response.context_data.get('registration'))
        self.assertTrue(tvu.exists() and tvu.count() == 1)
        self.assertEqual(tvu.first().amount, v.originalAmount)

    def test_vouchermakesitfree(self):
        '''
        Make a voucher larger than the price of the registration
        and ensure that this makes the registration free (and that
        it gets processed as such)
        '''
        updateConstant('vouchers__enableVouchers', True)
        s = self.create_series(pricingTier=self.defaultPricing)
        v = self.create_voucher(
            originalAmount=self.defaultPricing.getBasePrice() + 10,
            maxAmountPerUse=None,
        )

        response = self.register_to_check_voucher(v.voucherId,s)
        self.assertEqual(response.status_code,200)
        self.assertEqual(response.redirect_chain,[(reverse('showRegSummary'), 302)])
        self.assertEqual(response.context_data.get('totalPrice'), s.getBasePrice())
        self.assertEqual(response.context_data.get('netPrice'), 0)
        self.assertEqual(response.context_data.get('is_free'), True)
        self.assertEqual(response.context_data.get('total_voucher_amount'),s.getBasePrice())
        self.assertIn(v.name, response.context_data.get('voucher_names'))

        tr = response.context_data.get('registration')
        tvu = v.temporaryvoucheruse_set.filter(registration=tr)
        self.assertTrue(tvu.exists() and tvu.count() == 1)
        self.assertEqual(tvu.first().amount, s.getBasePrice())

        reg = tr.registration
        self.assertTrue(reg)
        self.assertEqual(reg.netPrice, 0)
        self.assertEqual(reg.totalPrice, s.getBasePrice())

        vu = v.voucheruse_set.filter(registration=reg)
        self.assertTrue(vu.exists() and vu.count() == 1)
        self.assertEqual(vu.first().amount, s.getBasePrice())



from django.conf.urls import url

from .views import createPaypalPayment, executePaypalPayment

urlpatterns = [
    url(r'^create_payment/$', createPaypalPayment, name='createPaypalPayment'),
    url(r'^execute_payment/$', executePaypalPayment, name='executePaypalPayment'),
]



from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from cms.models.pluginmodel import CMSPlugin
from cms.models.fields import PageField

import logging
from paypalrestsdk import Payment, Sale

from danceschool.core.models import PaymentRecord
from danceschool.core.constants import getConstant


# Define logger for this file
logger = logging.getLogger(__name__)


@python_2_unicode_compatible
class PaypalPaymentRecord(PaymentRecord):
    '''
    Keeps a local record of Paypal transactions so that they can be looked up
    using the REST API.
    '''

    paymentId = models.CharField(_('Paypal Payment ID'),max_length=50,unique=True)
    payerId = models.CharField(_('Paypal Payer ID'),max_length=50,null=True,blank=True)
    status = models.CharField(_('Current status'),max_length=30,null=True,blank=True)

    @property
    def methodName(self):
        return 'Paypal Express Checkout'

    @property
    def refundable(self):
        return True

    @property
    def recordId(self):
        '''
        Payment methods should override this if they keep their own unique identifiers.
        '''
        return self.paymentId

    def getPayment(self):
        return Payment.find(self.paymentId)

    def getSaleIds(self):
        ids = []
        payment = self.getPayment()
        for t in payment.transactions:
            for r in t.related_resources:
                if hasattr(r,'sale') and r.sale:
                    ids.append(r.sale.id)
        return ids

    def getRefundIds(self):
        ids = []
        payment = self.getPayment()
        for t in payment.transactions:
            for r in t.related_resources:
                if hasattr(r,'refund') and r.refund:
                    ids.append(r.refund.id)
        return ids

    @property
    def netAmountPaid(self):
        payment = self.getPayment()
        return sum([float(t.amount.total) for t in payment.transactions])

    def getPayerEmail(self):
        payment = self.getPayment
        return payment.payer.email

    def refund(self, amount=None):
        saleIds = self.getSaleIds()
        refundData = []

        leftToRefund = amount or 0
        for this_id in saleIds:
            # No need to continue if the full amount requested has been refunded
            if amount is not None and leftToRefund <= 0:
                break

            this_sale = Sale.find(this_id)

            if amount is not None:
                this_amount = min(float(this_sale.amount.total), leftToRefund)

                refund = this_sale.refund({
                    'amount': {
                        'total': '{0:.2f}'.format(this_amount),
                        'currency': getConstant('general__currencyCode'),
                    }
                })
            else:
                refund = this_sale.refund()

            if refund.success():
                logger.info('Refund successfully processed.')

                refundData.append({
                    'status': 'success',
                    'refund_id': refund.id,
                    'sale_id': refund.sale_id,
                    'refundAmount': float(refund.amount.total),

                    # This is (annoyingly) hard-coded for now because the Paypal REST API does
                    # not yet report fees in the event of a refund.  Hopefully this can be removed
                    # soon.
                    'fees': -1 * (
                        (float(this_sale.transaction_fee.value) - getConstant('paypal__fixedTransactionFee')) *
                        (float(refund.amount.total) / float(this_sale.amount.total))
                    ),
                })
                leftToRefund -= this_amount
            else:
                logger.error('Error processing refund.')
                refundData.append({'status': 'error', 'errors': refund.error})

        return refundData

    class Meta:
        verbose_name = _('Paypal payment record')
        verbose_name_plural = _('Payment records')


class PayNowFormModel(CMSPlugin):
    ''' This model holds options for instances of the GiftCertificateFormPlugin and the CartPaymentFormPlugin '''

    successPage = PageField(verbose_name=_('Success Page'),help_text=_('When the user returns to the site after a successful transaction, send them to this page.'),related_name='successPageFor')
    defaultAmount = models.FloatField(verbose_name=_('Default amount'),help_text=_('The initial value for gift certificate forms.'),default=0)

    def get_short_description(self):
        return self.plugin_type or self.id



from django.dispatch import receiver
from django.core.exceptions import ValidationError, ObjectDoesNotExist
from django.utils.translation import ugettext_lazy as _

from danceschool.core.signals import post_student_info, post_registration, apply_price_adjustments, get_customer_data, check_student_info
from danceschool.core.models import Customer, Series
from danceschool.core.constants import getConstant, REG_VALIDATION_STR

import logging

from .models import Voucher, TemporaryVoucherUse, VoucherUse
from .helpers import awardReferrers, ensureReferralVouchersExist


# Define logger for this file
logger = logging.getLogger(__name__)


@receiver(check_student_info)
def checkVoucherCode(sender,**kwargs):
    '''
    Check that the given voucher code is valid
    '''
    logger.debug('Signal to check RegistrationContactForm handled by vouchers app.')

    formData = kwargs.get('formData',{})
    request = kwargs.get('request',{})
    registration = kwargs.get('registration',None)
    session = getattr(request,'session',{}).get(REG_VALIDATION_STR,{})

    id = formData.get('gift','')
    first = formData.get('firstName')
    last = formData.get('lastName')
    email = formData.get('email')

    # Clean out the session data relating to vouchers so that we can revalidate it.
    session.pop('total_voucher_amount',0)
    session.pop('voucher_names',None)
    session.pop('gift',None)

    if id == '':
        return

    if not getConstant('vouchers__enableVouchers'):
        raise ValidationError({'gift': _('Vouchers are disabled.')})

    if session.get('gift','') != '':
        raise ValidationError({'gift': _('Can\'t have more than one voucher')})

    eventids = [x.event.id for x in registration.temporaryeventregistration_set.exclude(dropIn=True)]
    seriess = Series.objects.filter(id__in=eventids)

    obj = Voucher.objects.filter(voucherId=id).first()
    if not obj:
        raise ValidationError({'gift':_('Invalid Voucher Id')})
    else:
        customer = Customer.objects.filter(
            first_name=first,
            last_name=last,
            email=email).first()

        # This will raise any other errors that may be relevant
        try:
            obj.validateForCustomerAndSeriess(customer,seriess)
        except ValidationError as e:
            # Ensures that the error is applied to the correct field
            raise ValidationError({'gift': e})

    # If we got this far, then the voucher is determined to be valid, so the registration
    # can proceed with no errors.
    return


@receiver(post_student_info)
def applyVoucherCodeTemporarily(sender,**kwargs):
    '''
    When the core registration system creates a temporary registration with a voucher code,
    the voucher app looks for vouchers that match that code and creates TemporaryVoucherUse
    objects to keep track of the fact that the voucher may be used.
    '''
    logger.debug('Signal fired to apply temporary vouchers.')

    reg = kwargs.pop('registration')
    voucherId = reg.data.get('gift','')

    try:
        voucher = Voucher.objects.get(voucherId=voucherId)
    except ObjectDoesNotExist:
        logger.debug('No applicable vouchers found.')
        return

    tvu = TemporaryVoucherUse(voucher=voucher,registration=reg,amount=0)
    tvu.save()
    logger.debug('Temporary voucher use object created.')


@receiver(post_student_info)
def applyReferrerVouchersTemporarily(sender,**kwargs):
    '''
    Unlike voucher codes which have to be manually supplied, referrer discounts are
    automatically applied here, assuming that the referral program is enabled.
    '''

    # Only continue if the referral program is enabled
    if not getConstant('referrals__enableReferralProgram'):
        return

    logger.debug('Signal fired to temporarily apply referrer vouchers.')

    reg = kwargs.pop('registration')

    # Email address is unique for users, so use that
    try:
        c = Customer.objects.get(user__email=reg.email)
        vouchers = c.getReferralVouchers()
    except ObjectDoesNotExist:
        vouchers = None

    if not vouchers:
        logger.debug('No referral vouchers found.')
        return

    for v in vouchers:
        TemporaryVoucherUse(voucher=v,registration=reg,amount=0).save()


@receiver(apply_price_adjustments)
def applyTemporaryVouchers(sender,**kwargs):
    reg = kwargs.get('registration')
    price = kwargs.get('initial_price')

    logger.debug('Signal fired to apply temporary vouchers.')

    # Put referral vouchers first, so that they are applied last in the loop.
    referral_cat = getConstant('referrals__referrerCategory')

    tvus = list(reg.temporaryvoucheruse_set.filter(
        voucher__category=referral_cat
    )) + list(reg.temporaryvoucheruse_set.exclude(
        voucher__category=referral_cat
    ))

    if not tvus:
        logger.debug('No applicable vouchers found.')
        return ([],0)

    voucher_names = []
    total_voucher_amount = 0
    remaining_price = price

    while remaining_price > 0 and tvus:
        tvu = tvus.pop()

        if tvu.voucher.maxAmountPerUse:
            amount = min(tvu.voucher.amountLeft,tvu.voucher.maxAmountPerUse)
        else:
            amount = tvu.voucher.amountLeft
        amount = min(remaining_price,amount)
        tvu.amount = amount
        tvu.save()

        remaining_price -= amount
        voucher_names += [tvu.voucher.name]
        total_voucher_amount += amount

    return (voucher_names,total_voucher_amount)


@receiver(post_registration)
def applyVoucherCodesFinal(sender,**kwargs):
    '''
    Once a registration has been completed, vouchers are used and referrers are awarded
    '''
    logger.debug('Signal fired to mark voucher codes as applied.')

    finalReg = kwargs.pop('registration')
    tr = finalReg.temporaryRegistration

    tvus = TemporaryVoucherUse.objects.filter(registration=tr)

    for tvu in tvus:
        vu = VoucherUse(voucher=tvu.voucher,registration=finalReg,amount=tvu.amount)
        vu.save()
        if getConstant('referrals__enableReferralProgram'):
            awardReferrers(vu)


@receiver(get_customer_data)
def provideCustomerReferralCode(sender,**kwargs):
    '''
    If the vouchers app is installed and referrals are enabled, then the customer's profile page can show their voucher referral code.
    '''
    customer = kwargs.pop('customer')
    if getConstant('vouchers__enableVouchers') and getConstant('referrals__enableReferralProgram'):
        vrd = ensureReferralVouchersExist(customer)

        return {
            'referralVoucherId': vrd.referreeVoucher.voucherId
        }



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-02 22:46
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('core', '0016_auto_20170830_1159'),
    ]

    operations = [
        migrations.CreateModel(
            name='InstructorAvailabilitySlot',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('startTime', models.DateTimeField(verbose_name='Start time')),
                ('duration', models.PositiveSmallIntegerField(default=30, verbose_name='Slot duration (minutes)')),
                ('status', models.CharField(choices=[('A', 'Available'), ('B', 'Booked'), ('T', 'Tentative Booking'), ('U', 'Unavailable')], default='A', max_length=1)),
                ('creationDate', models.DateTimeField(auto_now_add=True)),
                ('modifiedDate', models.DateTimeField(auto_now=True)),
                ('eventRegistration', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='privateLessonSlots', to='core.EventRegistration', verbose_name='Final event registration')),
                ('instructor', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='core.Instructor', verbose_name='Instructor')),
            ],
            options={'ordering': ('-startTime', 'instructor__lastName', 'instructor__firstName'), 'permissions': (('edit_own_availability', "Can edit one's own private lesson availability."), ('edit_others_availability', "Can edit other instructors' private lesson availability.")), 'verbose_name': 'Private lesson availability slot', 'verbose_name_plural': 'Private lesson availability slots'},
        ),
        migrations.CreateModel(
            name='InstructorPrivateLessonDetails',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('couples', models.BooleanField(default=True, verbose_name='Private lessons for couples')),
                ('smallGroups', models.BooleanField(default=True, verbose_name='Private lessons for small groups')),
                ('defaultPricingTier', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='core.PricingTier', verbose_name='Default Pricing Tier')),
                ('instructor', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='core.Instructor')),
                ('roles', models.ManyToManyField(blank=True, to='core.DanceRole')),
            ],
            options={'ordering': ('instructor__lastName', 'instructor__firstName'), 'verbose_name': 'Instructor private lesson details', 'verbose_name_plural': "Instructors' private lesson details"},
        ),
        migrations.CreateModel(
            name='PrivateLessonEvent',
            fields=[
                ('event_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.Event')),
                ('participants', models.PositiveSmallIntegerField(blank=True, default=1, null=True, verbose_name='Expected # of Participants')),
                ('comments', models.TextField(blank=True, help_text='For internal use and recordkeeping.', null=True, verbose_name='Comments/Notes')),
                ('pricingTier', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='core.PricingTier', verbose_name='Pricing Tier')),
            ],
            options={'abstract': False, 'verbose_name': 'Private lesson', 'verbose_name_plural': 'Private lessons', 'permissions': (('view_others_lessons', 'Can view scheduled private lessons for all instructors'),),},
            bases=('core.event',),
        ),
        migrations.CreateModel(
            name='PrivateLessonCustomer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('customer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='core.Customer', verbose_name='Customer')),
                ('lesson', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='private_lessons.PrivateLessonEvent', verbose_name='Lesson')),
            ],
            options={
                'verbose_name': 'Private lesson customer',
                'verbose_name_plural': 'Private lesson customers',
            },
        ),
        migrations.AddField(
            model_name='instructoravailabilityslot',
            name='lessonEvent',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='private_lessons.PrivateLessonEvent', verbose_name='Scheduled lesson'),
        ),
        migrations.AddField(
            model_name='instructoravailabilityslot',
            name='location',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.Location', verbose_name='Location'),
        ),
        migrations.AddField(
            model_name='instructoravailabilityslot',
            name='pricingTier',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.PricingTier', verbose_name='Pricing Tier'),
        ),
        migrations.AddField(
            model_name='instructoravailabilityslot',
            name='temporaryEventRegistration',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='privateLessonSlots', to='core.TemporaryEventRegistration', verbose_name='Temporary event registration'),
        ),
        migrations.AlterUniqueTogether(
            name='privatelessoncustomer',
            unique_together=set([('customer', 'lesson')]),
        ),
    ]



from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from .models import NewsItem


class NewsItemAdmin(admin.ModelAdmin):
    list_display = ('title','alert','pinThis','publicationDate','modifiedDate')
    list_display_links = ('title',)
    list_filter = ('publicationDate','modifiedDate','alert','pinThis')
    readonly_fields = ('modifiedDate',)

    fieldsets = (
        (None, {
            'fields': ('title','content',('alert','pinThis'),('draft','hideThis'),)
        }),
        (_('Dates'),{
            'classes': ('collapse',),
            'fields': ('publicationDate','modifiedDate'),
        }),
    )


admin.site.register(NewsItem, NewsItemAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-08-09 18:28
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0011_auto_20170720_1945'),
    ]

    operations = [
        migrations.CreateModel(
            name='SeriesCategory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Category name will be displayed.', max_length=100, unique=True, verbose_name='Name')),
                ('description', models.TextField(blank=True, help_text='Add an optional description.', null=True, verbose_name='Description')),
                ('slug', models.SlugField(help_text='This slug is used primarily for custom templates in registration, if the category is shown separately on the registration page.  You can override the default.', verbose_name='Slug')),
                ('separateOnRegistrationPage', models.BooleanField(default=False, verbose_name='Show category separately on registration page')),
            ],
            options={
                'verbose_name_plural': 'Series categories',
                'verbose_name': 'Series category',
            },
        ),
        migrations.AlterModelOptions(
            name='pricingtier',
            options={'verbose_name': 'Pricing tier', 'verbose_name_plural': 'Pricing tiers'},
        ),
        migrations.RemoveField(
            model_name='series',
            name='special',
        ),
        migrations.AddField(
            model_name='publiceventcategory',
            name='separateOnRegistrationPage',
            field=models.BooleanField(default=False, verbose_name='Show category separately on registration page'),
        ),
        migrations.AddField(
            model_name='publiceventcategory',
            name='slug',
            field=models.SlugField(default='slug', help_text='This slug is used primarily for custom templates in registration, if the category is shown separately on the registration page.  You can override the default.', verbose_name='Slug'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='series',
            name='allowDropins',
            field=models.BooleanField(default=False, help_text='If checked, then all staff will be able to register students as drop-ins.', verbose_name='Allow class drop-ins'),
        ),
        migrations.AddField(
            model_name='series',
            name='category',
            field=models.ForeignKey(blank=True, help_text='Custom series categories may be used to display special series (e.g. one-offs, visiting instructors) separately on your registration page.', null=True, on_delete=django.db.models.deletion.CASCADE, to='core.SeriesCategory', verbose_name='Series category (optional)'),
        ),
        migrations.AlterField(
            model_name='publicevent',
            name='category',
            field=models.ForeignKey(blank=True, help_text='Custom event categories may be used to display special types of events (e.g. practice sessions) separately on your registration page.  They may also be displayed in different colors on the public calendar.', null=True, on_delete=django.db.models.deletion.CASCADE, to='core.PublicEventCategory', verbose_name='Category (optional)'),
        ),
    ]



from django.apps import AppConfig


class DiscountsAppConfig(AppConfig):
    name = 'danceschool.discounts'
    verbose_name = 'Discount System Functions'

    def ready(self):
        # This ensures that the signal receivers are loaded
        from . import handlers



import uuid
from django.utils.translation import ugettext_lazy as _

from danceschool.core.models import DanceTypeLevel, ClassDescription
from danceschool.core.constants import getConstant

from .models import Voucher, VoucherCategory, ClassVoucher, CustomerVoucher, VoucherReferralDiscount, VoucherCredit, VoucherReferralDiscountUse


def generateUniqueVoucherId(prefix):
    uid = uuid.uuid4()
    result = prefix + uid.hex[0:8].upper()
    objs = Voucher.objects.filter(voucherId=result)

    while len(objs) > 0:
        result = prefix + uid.hex[0:8]
        objs = Voucher.objects.filter(voucherId=result)

    return result


def createReferreeVoucher(name,amountPerUse):
    voucherId = generateUniqueVoucherId(getConstant('referrals__voucherPrefix'))
    category = getConstant('referrals__refereeCategory')
    originalAmount = 1e9
    maxAmountPerUse = amountPerUse
    singleUse = False
    forFirstTimeCustomersOnly = True
    expirationDate = None
    disabled = False

    voucher = Voucher(
        voucherId=voucherId,name=name,
        category=category,
        originalAmount=originalAmount,
        maxAmountPerUse=maxAmountPerUse,
        singleUse=singleUse,
        forFirstTimeCustomersOnly=forFirstTimeCustomersOnly,
        expirationDate=expirationDate,
        disabled=disabled
    )
    voucher.save()

    # Find all beginner classes
    dts = DanceTypeLevel.objects.filter(name="Beginner",danceType__name="Lindy Hop")
    classes = ClassDescription.objects.filter(danceTypeLevel=dts.first())
    for c in classes:
        cv = ClassVoucher(voucher=voucher,classDescription=c)
        cv.save()
    return voucher


def createReferrerVoucher(customer):
    voucherId = generateUniqueVoucherId(getConstant('referrals__voucherPrefix'))
    category = getConstant('referrals__referrerCategory')
    originalAmount = 0
    maxAmountPerUse = None
    singleUse = False
    forFirstTimeCustomersOnly = False
    expirationDate = None
    disabled = False
    name = _("Referral Bonus for %s, %s" % (customer.fullName,customer.email))

    voucher = Voucher(
        voucherId=voucherId,name=name,
        category=category,
        originalAmount=originalAmount,
        maxAmountPerUse=maxAmountPerUse,
        singleUse=singleUse,
        forFirstTimeCustomersOnly=forFirstTimeCustomersOnly,
        expirationDate=expirationDate,
        disabled=disabled
    )
    voucher.save()

    cv = CustomerVoucher(customer=customer,voucher=voucher)
    cv.save()

    return voucher


def referralVoucherExists(customer):

    for cv in CustomerVoucher.objects.filter(customer=customer):
        vrd = VoucherReferralDiscount.objects.filter(referrerVoucher=cv.voucher).first()
        if vrd:
            return vrd

def ensureReferralVouchersExist(customer):
    # is there a referral voucher for this person?
    # Find all CustomerVouchers for this person
    vrd = referralVoucherExists(customer)

    referreeDiscount = getConstant('referrals__refereeDiscount')
    referrerDiscount = getConstant('referrals__referrerDiscount')

    if vrd:
        vrd.amount = referrerDiscount
        vrd.save()

        vrd.referreeVoucher.maxAmountPerUse = referreeDiscount
        vrd.referreeVoucher.save()
    else:
        name = _('Referral: %s' % customer.fullName)

        # create the referree voucher
        referreeVoucher = createReferreeVoucher(name, referreeDiscount)

        # create the referrer voucher
        referrerVoucher = createReferrerVoucher(customer)

        # create the thing that ties them together
        vrd = VoucherReferralDiscount.objects.get_or_create(
            referrerVoucher=referrerVoucher,
            referreeVoucher=referreeVoucher,
            referrerBonus=referrerDiscount
        )

    return vrd


def awardReferrers(voucherUse):
    rds = VoucherReferralDiscount.objects.filter(referreeVoucher=voucherUse.voucher)

    for rd in rds:
        vc = VoucherCredit(
            voucher=rd.referrerVoucher,
            amount=rd.referrerBonus,
            description=_("Referral from " + str(rd.referreeVoucher)),
        )
        vc.save()
        vrdu = VoucherReferralDiscountUse(
            voucherReferralDiscount=rd,
            voucherUse=voucherUse,
            voucherCredit=vc
        )
        vrdu.save()



from django.conf.urls import url

from . import stats
from .views import SchoolStatsView

urlpatterns = [
    # These are stats CSV queries that will be eventually moved to the management app
    url(r'^$', SchoolStatsView.as_view(),name='schoolStatsView'),

    url(r'^monthlyperformance/csv/$', stats.MonthlyPerformanceCSV,name='MonthlyPerformanceCSV'),
    url(r'^monthlyperformance/json/$', stats.MonthlyPerformanceJSON,name='MonthlyPerformanceJSON'),

    url(r'^classcounthistogram/csv/$',stats.ClassCountHistogramCSV,name='ClassCountHistogramCSV'),
    url(r'^classcounthistogram/json/$',stats.ClassCountHistogramJSON,name='ClassCountHistogramJSON'),

    url(r'^averagesbyclasstype/csv/$',stats.AveragesByClassTypeCSV,name='AveragesByClassTypeCSV'),
    url(r'^averagesbyclasstype/json/$',stats.AveragesByClassTypeJSON,name='AveragesByClassTypeJSON'),

    url(r'^classtypemonthly/json/$',stats.ClassTypeMonthlyJSON,name='ClassTypeMonthlyJSON'),

    url(r'^locationperformance/csv/$',stats.LocationPerformanceCSV,name='LocationPerformanceCSV'),
    url(r'^locationperformance/json/$',stats.LocationPerformanceJSON,name='LocationPerformanceJSON'),

    url(r'^advanceregistration/json/$',stats.AdvanceRegistrationDaysJSON,name='AdvanceRegistrationDaysJSON'),
    url(r'^registrationhours/json/$',stats.RegistrationHoursJSON,name='RegistrationHoursJSON'),
    url(r'^multiregistration/json/$',stats.MultiRegistrationJSON,name='MultiRegistrationJSON'),

    url(r'^registrationtypeaverages/json/$',stats.RegistrationTypeAveragesJSON,name='RegistrationTypeAveragesJSON'),
    url(r'^referralcounts/json/$',stats.RegistrationReferralCountsJSON,name='RegistrationReferralCountsJSON'),

    url(r'^bestcustomers/json/$',stats.getBestCustomersJSON,name='bestCustomersJSON'),
]



from django.core.management.base import BaseCommand

from danceschool.core.models import Series


class Command(BaseCommand):
    help = 'Update the registration status for series and events that should be opened/closed based on the current time'

    def handle(self, *args, **options):
        self.stdout.write('Checking and updating registration status of all opened class series.')

        open_series = Series.objects.filter(**{'registrationOpen': True})

        for series in open_series:
            series.updateRegistrationStatus()
        self.stdout.write('done.')



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-08-09 00:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('discounts', '0003_auto_20170724_2114'),
    ]

    operations = [
        migrations.AddField(
            model_name='discountcombo',
            name='daysInAdvanceRequired',
            field=models.PositiveIntegerField(blank=True, help_text='For this discount to apply, all components must be satisfied by events that begin this many days in the future (measured from midnight of the date of registration).  Leave blank for no restriction.', null=True, verbose_name='Must register __ days in advance'),
        ),
        migrations.AddField(
            model_name='discountcombo',
            name='expirationDate',
            field=models.DateTimeField(blank=True, help_text='Leave blank for no expiration.', null=True, verbose_name='Expiration Date'),
        ),
    ]



from django.conf.urls import url

from .feeds import EventFeed, json_event_feed
from .views import addPrivateEvent, PrivateCalendarView

urlpatterns = [
    # These are the calendar feeds.
    url(r'^feed/json/$', json_event_feed, name='jsonPrivateCalendarFeed'),
    url(r'^feed/json/location/(?P<location_id>[0-9]+)/(?P<room_id>[0-9]+)/$', json_event_feed, name='jsonPrivateCalendarFeed'),
    url(r'^feed/json/location/(?P<location_id>[0-9]+)/$', json_event_feed, name='jsonPrivateCalendarFeed'),
    url(r'^feed/(?P<instructorFeedKey>[\w\-_]+)/$', EventFeed(), name='privateCalendarFeed'),

    url(r'^calendar/$', PrivateCalendarView.as_view(), name='privateCalendar'),

    url(r'^add/$',addPrivateEvent,name='addPrivateEvent'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-09 04:24
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0016_auto_20170830_1159'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cashpaymentrecord',
            name='collectedByUser',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='collectedcashpayments', to=settings.AUTH_USER_MODEL, verbose_name='Collected by user'),
        ),
        migrations.AlterField(
            model_name='classdescription',
            name='danceTypeLevel',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.SET_DEFAULT, to='core.DanceTypeLevel', verbose_name='Dance Type & Level'),
        ),
        migrations.AlterField(
            model_name='customer',
            name='user',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='User account'),
        ),
        migrations.AlterField(
            model_name='emailtemplate',
            name='groupRequired',
            field=models.ForeignKey(blank=True, help_text='Some templates should only be visible to some users.', null=True, on_delete=django.db.models.deletion.SET_NULL, to='auth.Group', verbose_name='Group permissions required to use.'),
        ),
        migrations.AlterField(
            model_name='event',
            name='location',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.Location', verbose_name='Location'),
        ),
        migrations.AlterField(
            model_name='event',
            name='room',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.Room', verbose_name='Room'),
        ),
        migrations.AlterField(
            model_name='event',
            name='submissionUser',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='eventsubmissions', to=settings.AUTH_USER_MODEL, verbose_name='Submitted by user'),
        ),
        migrations.AlterField(
            model_name='eventlistpluginmodel',
            name='location',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.Location', verbose_name='Limit to location'),
        ),
        migrations.AlterField(
            model_name='eventregistration',
            name='role',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.DanceRole', verbose_name='Dance role'),
        ),
        migrations.AlterField(
            model_name='eventstaffmember',
            name='category',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.EventStaffCategory', verbose_name='Category'),
        ),
        migrations.AlterField(
            model_name='eventstaffmember',
            name='replacedStaffMember',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='replacementFor', to='core.EventStaffMember', verbose_name='Replacement for'),
        ),
        migrations.AlterField(
            model_name='eventstaffmember',
            name='submissionUser',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='Submission User'),
        ),
        migrations.AlterField(
            model_name='instructorlistpluginmodel',
            name='imageThumbnail',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='filer.ThumbnailOption', verbose_name='Image thumbnail option'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='collectedByUser',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='collectedinvoices', to=settings.AUTH_USER_MODEL, verbose_name='Collected by user'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='finalRegistration',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.Registration', verbose_name='Registration'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='submissionUser',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='submittedinvoices', to=settings.AUTH_USER_MODEL, verbose_name='Registered by user'),
        ),
        migrations.AlterField(
            model_name='invoiceitem',
            name='finalEventRegistration',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.EventRegistration', verbose_name='Event registration'),
        ),
        migrations.AlterField(
            model_name='paymentrecord',
            name='invoice',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.Invoice', verbose_name='Invoice'),
        ),
        migrations.AlterField(
            model_name='paymentrecord',
            name='submissionUser',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='payments_submitted', to=settings.AUTH_USER_MODEL, verbose_name='Submission user'),
        ),
        migrations.AlterField(
            model_name='publicevent',
            name='category',
            field=models.ForeignKey(blank=True, help_text='Custom event categories may be used to display special types of events (e.g. practice sessions) separately on your registration page.  They may also be displayed in different colors on the public calendar.', null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.PublicEventCategory', verbose_name='Category (optional)'),
        ),
        migrations.AlterField(
            model_name='publicevent',
            name='pricingTier',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.PricingTier', verbose_name='Pricing Tier'),
        ),
        migrations.AlterField(
            model_name='series',
            name='category',
            field=models.ForeignKey(blank=True, help_text='Custom series categories may be used to display special series (e.g. one-offs, visiting instructors) separately on your registration page.', null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.SeriesCategory', verbose_name='Series category (optional)'),
        ),
        migrations.AlterField(
            model_name='series',
            name='classDescription',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.ClassDescription', verbose_name='Class description'),
        ),
        migrations.AlterField(
            model_name='series',
            name='pricingTier',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='core.PricingTier', verbose_name='Pricing tier'),
        ),
        migrations.AlterField(
            model_name='staffmember',
            name='userAccount',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='User account'),
        ),
        migrations.AlterField(
            model_name='temporaryeventregistration',
            name='role',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.DanceRole', verbose_name='Dance role'),
        ),
        migrations.AlterField(
            model_name='temporaryregistration',
            name='submissionUser',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='submittedtemporaryregistrations', to=settings.AUTH_USER_MODEL, verbose_name='registered by user'),
        ),
    ]



# Give this app a custom verbose name to avoid confusion
from django.apps import AppConfig
from django.conf import settings


class SquareAppConfig(AppConfig):
    name = 'danceschool.payments.square'
    verbose_name = 'Square Functions'

    def ready(self):
        import squareconnect
        squareconnect.configuration.access_token = getattr(settings,'SQUARE_ACCESS_TOKEN','')



from django.contrib import admin
from django.utils.html import format_html
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from .models import PaypalPaymentRecord


class PaypalPaymentRecordAdmin(admin.ModelAdmin):

    def get_admin_change_link(self,app_label, model_name, obj_id, name):
        url = reverse('admin:%s_%s_change' % (app_label, model_name),
                      args=(obj_id,))
        return format_html('<a href="%s">%s</a>' % (
            url, str(name)
        ))

    def invoiceLink(self,item):
        if item.invoice:
            i = item.invoice
            return self.get_admin_change_link('core','invoice',i.id,i.id)
    invoiceLink.allow_tags = True
    invoiceLink.short_description = _('Registration invoice')

    list_display = ['paymentId','payerId','status','invoiceLink']
    list_filter = ['status','creationDate','modifiedDate']
    search_fields = ['paymentId','payerId','invoice__id']

    ordering = ['-modifiedDate',]
    readonly_fields = ['paymentId','payerId','status','creationDate','modifiedDate','invoiceLink']

    fieldsets = (
        (_('Basic Information'), {
            'fields': ('paymentId','payerId','status','invoiceLink'),
        }),
        (_('Dates'), {
            'fields': ('creationDate','modifiedDate'),
        }),
    )


admin.site.register(PaypalPaymentRecord, PaypalPaymentRecordAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-24 21:03
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('vouchers', '0002_auto_20170717_1642'),
    ]

    operations = [
        migrations.RenameField(
            model_name='voucher',
            old_name='type',
            new_name='description',
        ),
        migrations.AlterField(
            model_name='voucher',
            name='description',
            field=models.CharField(blank=True, help_text='For internal use only', max_length=200, null=True, verbose_name='Description (optional)'),
        ),
        migrations.AlterField(
            model_name='voucher',
            name='disabled',
            field=models.BooleanField(default=False, help_text='Check this box to disable the voucher entirely.', verbose_name='Voucher Disabled'),
        ),
        migrations.AlterField(
            model_name='voucher',
            name='maxAmountPerUse',
            field=models.FloatField(blank=True, help_text='If specified, this will limit the size of a repeated-use voucher.  If unspecified, there is no limit.  Be sure to specify this for publicly advertised voucher codes.', null=True, validators=[django.core.validators.MinValueValidator(0)], verbose_name='Max. Amount Per Use'),
        ),
        migrations.AlterField(
            model_name='voucher',
            name='name',
            field=models.CharField(help_text='Give a descriptive name that will be used when a customer applies the voucher.', max_length=80, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='voucher',
            name='singleUse',
            field=models.BooleanField(default=False, verbose_name='Single Use'),
        ),
    ]



from django.contrib import admin
from adminsortable2.admin import SortableAdminMixin
from cms.admin.placeholderadmin import FrontendEditableAdminMixin

from .models import FAQCategory, FAQ


class FAQCategoryAdmin(SortableAdminMixin, admin.ModelAdmin):
    list_display = ('name',)


class FAQAdmin(SortableAdminMixin, FrontendEditableAdminMixin, admin.ModelAdmin):
    list_display = ('question','category','draft')
    list_filter = ('category','draft')
    lsit_editable = ('category','draft')


admin.site.register(FAQCategory, FAQCategoryAdmin)
admin.site.register(FAQ, FAQAdmin)



from django.http import JsonResponse, HttpResponseRedirect, HttpResponseBadRequest
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone

from danceschool.core.constants import getConstant, INVOICE_VALIDATION_STR
from danceschool.core.models import Invoice, TemporaryRegistration

from .models import StripeCharge

import stripe
import logging
from datetime import timedelta


# Define logger for this file
logger = logging.getLogger(__name__)


def handle_stripe_checkout(request):

    logger.info('Received request for Stripe Checkout payment.')

    stripeToken = request.POST.get('stripeToken')
    stripeEmail = request.POST.get('stripeEmail')
    submissionUserId = request.POST.get('submissionUser')
    amount = request.POST.get('stripeAmount')
    invoice_id = request.POST.get('invoice_id')
    tr_id = request.POST.get('reg_id')
    transactionType = request.POST.get('transaction_type')
    taxable = request.POST.get('taxable', False)
    addSessionInfo = request.POST.get('addSessionInfo',False)
    customizeUrl = request.POST.get('customizeUrl')
    successUrl = request.POST.get('successUrl',reverse('registration'))

    # Parse if a specific submission user is indicated
    submissionUser = None
    if submissionUserId:
        try:
            submissionUser = User.objects.get(id=int(submissionUserId))
        except (ValueError,ObjectDoesNotExist):
            logger.warning('Invalid user passed, submissionUser will not be recorded.')

    # If a specific amount to pay has been passed, then allow payment
    # of that amount.
    if amount:
        try:
            if isinstance(amount,list):
                amount = float(amount[0])
            else:
                amount = float(amount)
        except ValueError:
            logger.error('Invalid amount passed')
            return HttpResponseBadRequest()

    # If the details of this transaction are to be entered into session info, then
    # the view must redirect to an interim URL (e.g. for gift certificates) that
    # handles that data
    if addSessionInfo and not customizeUrl:
        logger.error('Request to pass session info without specifying interim URL.')
        return HttpResponseBadRequest()

    try:
        # Invoice transactions are usually payment on an existing invoice.
        if invoice_id:
            this_invoice = Invoice.objects.get(id=invoice_id)
            this_description = _('Invoice Payment: %s' % this_invoice.id)
            if not amount:
                amount = this_invoice.outstandingBalance
        # This is typical of payment at the time of registration
        elif tr_id:
            tr = TemporaryRegistration.objects.get(id=int(tr_id))
            tr.expirationDate = timezone.now() + timedelta(minutes=getConstant('registration__sessionExpiryMinutes'))
            tr.save()
            this_invoice = Invoice.get_or_create_from_registration(tr, submissionUser=submissionUser)
            this_description = _('Registration Payment: #%s' % tr_id)
            if not amount:
                amount = this_invoice.outstandingBalance
        # All other transactions require both a transaction type and an amount to be specified
        elif not transactionType or not amount:
            logger.error('Insufficient information passed to createPaypalPayment view.')
            raise ValueError
        else:
            # Gift certificates automatically get a nicer invoice description
            if transactionType == 'Gift Certificate':
                this_description = _('Gift Certificate Purchase')
            else:
                this_description = transactionType
            this_invoice = Invoice.create_from_item(
                float(amount),
                this_description,
                submissionUser=submissionUser,
                calculate_taxes=(taxable is not False),
                transactionType=transactionType,
            )
    except (ValueError, ObjectDoesNotExist) as e:
        logger.error('Invalid registration information passed to handle_stripe_checkout view: (%s, %s, %s)' % (invoice_id, tr_id, amount))
        logger.error(e)
        return HttpResponseBadRequest()

    this_total = int(min(this_invoice.outstandingBalance, amount) * 100)
    charge = None

    try:
        # Use Stripe's library to make requests...
        charge = stripe.Charge.create(
            amount=this_total,
            currency=getConstant('general__currencyCode'),
            description=this_description,
            source=stripeToken,
        )

    except stripe.error.CardError as e:
        # Since it's a decline, stripe.error.CardError will be caught
        body = e.json_body
        err = body['error']
        logger.error('Stripe CardError %s: %s' % (e.http_status,err))
    except stripe.error.RateLimitError as e:
        # Too many requests made to the API too quickly
        body = e.json_body
        err = body['error']
        logger.error('Stripe RateLimitError %s: %s' % (e.http_status,err))
    except stripe.error.InvalidRequestError as e:
        # Invalid parameters were supplied to Stripe's API
        body = e.json_body
        err = body['error']
        logger.error('Stripe InvalidRequestError %s: %s' % (e.http_status,err))
    except stripe.error.AuthenticationError as e:
        # Authentication with Stripe's API failed
        # (maybe you changed API keys recently)
        body = e.json_body
        err = body['error']
        logger.error('Stripe AuthenticationError %s: %s' % (e.http_status,err))
    except stripe.error.APIConnectionError as e:
        # Network communication with Stripe failed
        body = e.json_body
        err = body['error']
        logger.error('Stripe APIConnectionError %s: %s' % (e.http_status,err))
    except stripe.error.StripeError as e:
        # Display a very generic error to the user, and maybe send
        # yourself an email
        body = e.json_body
        err = body['error']
        logger.error('Stripe StripeError %s: %s' % (e.http_status,err))

    if charge:
        StripeCharge.objects.create(
            chargeId=charge.id,
            status=charge.status,
            submissionUser=submissionUser,
            invoice=this_invoice,
        )

        # To determine the fees applied, we also need to get the balanceTransaction
        # that reports them.
        balanceTransaction = stripe.BalanceTransaction.retrieve(charge.balance_transaction)

        this_invoice.processPayment(
            amount=charge.amount / 100,
            fees=balanceTransaction.fee / 100,
            paidOnline=True,
            methodName='Stripe Checkout',
            methodTxn=charge.id,
            submissionUser=submissionUser,
            notify=stripeEmail,
        )

        if addSessionInfo:
            paymentSession = request.session.get(INVOICE_VALIDATION_STR, {})

            paymentSession.update({
                'invoiceID': str(this_invoice.id),
                'amount': charge.amount / 100,
                'successUrl': successUrl,
            })
            request.session[INVOICE_VALIDATION_STR] = paymentSession
            return HttpResponseRedirect(customizeUrl)

        return HttpResponseRedirect(successUrl)

    else:
        # TODO: Improve error handling.
        return JsonResponse(err)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-17 16:42
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('discounts', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='discountcombo',
            options={'verbose_name': 'Discount', 'verbose_name_plural': 'Discounts'},
        ),
        migrations.AlterModelOptions(
            name='discountcombocomponent',
            options={'verbose_name': 'Required component of discount', 'verbose_name_plural': 'Required components of discount'},
        ),
        migrations.AlterModelOptions(
            name='pointgroup',
            options={'verbose_name': 'Point group type', 'verbose_name_plural': 'Point group types'},
        ),
        migrations.AlterModelOptions(
            name='pricingtiergroup',
            options={'verbose_name': 'Pricing tier point group', 'verbose_name_plural': 'Pricing tier point groups'},
        ),
        migrations.AlterModelOptions(
            name='registrationdiscount',
            options={'verbose_name': 'Discount applied to registration', 'verbose_name_plural': 'Discounts applied to registrations'},
        ),
        migrations.AlterModelOptions(
            name='temporaryregistrationdiscount',
            options={'verbose_name': 'Discount applied to temporary registration', 'verbose_name_plural': 'Discounts applied to temporary registrations'},
        ),
        migrations.AlterField(
            model_name='discountcombo',
            name='active',
            field=models.BooleanField(default=True, help_text="Uncheck this to 'turn off' this discount", verbose_name='Active'),
        ),
        migrations.AlterField(
            model_name='discountcombo',
            name='discountType',
            field=models.CharField(choices=[('F', 'Exact Specified Price'), ('D', 'Dollar Discount from Regular Price'), ('P', 'Percentage Discount from Regular Price'), ('A', 'Free Add-on Item (Can be combined with other discounts)')], help_text="Is this a flat price, a dollar amount discount, a 'percentage off' discount, or a free add-on?", max_length=1, verbose_name='Discount type'),
        ),
        migrations.AlterField(
            model_name='discountcombo',
            name='dollarDiscount',
            field=models.FloatField(blank=True, help_text="This amount will be subtracted from the customer's total (in currency units, e.g. dollars).", null=True, validators=[django.core.validators.MinValueValidator(0)], verbose_name='Amount of dollar discount'),
        ),
        migrations.AlterField(
            model_name='discountcombo',
            name='name',
            field=models.CharField(help_text="Give this discount combination a name (e.g. 'Two 4-week series')", max_length=50, unique=True, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='discountcombo',
            name='percentDiscount',
            field=models.FloatField(blank=True, help_text="This percentage will be subtracted from the customer's total.", null=True, validators=[django.core.validators.MinValueValidator(0)], verbose_name='Amount of percentage discount'),
        ),
        migrations.AlterField(
            model_name='discountcombo',
            name='percentUniversallyApplied',
            field=models.BooleanField(default=False, help_text='If checked, then the percentage discount will be applied to all items in the order, not just the items that qualify the order for this discount combination (e.g. 20% off all registrations of three or more classes.', verbose_name='Percentage discount is univerally applied'),
        ),
        migrations.AlterField(
            model_name='discountcombocomponent',
            name='discountCombo',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='discounts.DiscountCombo', verbose_name='Discount'),
        ),
        migrations.AlterField(
            model_name='discountcombocomponent',
            name='level',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='core.DanceTypeLevel', verbose_name='Dance type/level'),
        ),
        migrations.AlterField(
            model_name='discountcombocomponent',
            name='quantity',
            field=models.PositiveSmallIntegerField(default=1, verbose_name='Quantity'),
        ),
        migrations.AlterField(
            model_name='discountcombocomponent',
            name='weekday',
            field=models.PositiveSmallIntegerField(blank=True, choices=[(None, ''), (0, 'Monday'), (1, 'Tuesday'), (2, 'Wednesday'), (3, 'Thursday'), (4, 'Friday'), (5, 'Saturday'), (6, 'Sunday')], help_text='Leave this blank for no restriction on days of the week', null=True, verbose_name='Weekday'),
        ),
        migrations.AlterField(
            model_name='pointgroup',
            name='name',
            field=models.CharField(help_text="Give this pricing tier point group a name (e.g. 'Regular Series Class Hours')", max_length=50, unique=True, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='pricingtiergroup',
            name='group',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='discounts.PointGroup', verbose_name='Point group'),
        ),
        migrations.AlterField(
            model_name='pricingtiergroup',
            name='points',
            field=models.PositiveIntegerField(default=0, verbose_name='# of Points'),
        ),
        migrations.AlterField(
            model_name='pricingtiergroup',
            name='pricingTier',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='core.PricingTier', verbose_name='Pricing tier'),
        ),
        migrations.AlterField(
            model_name='registrationdiscount',
            name='discount',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='discounts.DiscountCombo', verbose_name='Discount'),
        ),
        migrations.AlterField(
            model_name='registrationdiscount',
            name='registration',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='core.Registration', verbose_name='Registration'),
        ),
        migrations.AlterField(
            model_name='temporaryregistrationdiscount',
            name='discount',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='discounts.DiscountCombo', verbose_name='Discount'),
        ),
        migrations.AlterField(
            model_name='temporaryregistrationdiscount',
            name='registration',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='core.TemporaryRegistration', verbose_name='Temporary registration'),
        ),
    ]



from django.conf.urls import url

from .views import GiftCertificateCustomizeView
from .stats import popularVouchersJSON, voucherFrequencyJSON

urlpatterns = [
    url(r'^gift_certificate/customize/$', GiftCertificateCustomizeView.as_view(), name='customizeGiftCertificate'),
    url(r'^popularvouchers/json/$', popularVouchersJSON, name='popularVouchersJSON'),
    url(r'^voucherfrequency/json/$', voucherFrequencyJSON, name='voucherFrequencyJSON'),
]



from django.conf.urls import url

from .stats import popularDiscountsJSON, discountFrequencyJSON

urlpatterns = [
    url(r'^populardiscounts/json/$', popularDiscountsJSON, name='popularDiscountsJSON'),
    url(r'^discountfrequency/json/$', discountFrequencyJSON, name='discountFrequencyJSON'),
]



# Give this app a custom verbose name to avoid confusion
from django.apps import AppConfig


class VoucherAppConfig(AppConfig):
    name = 'danceschool.vouchers'
    verbose_name = 'Voucher Functions'

    def ready(self):
        from danceschool.core.models import Customer
        from danceschool.core.constants import getConstant
        from .models import CustomerVoucher

        def creditsAvailable(customer):
            cvs = CustomerVoucher.objects.filter(customer=customer)
            amount = 0
            for cv in cvs:
                amount += cv.voucher.amountLeft

            return amount

        def getCustomerVouchers(customer):
            cvs = CustomerVoucher.objects.filter(customer=customer)
            return [cv.voucher for cv in cvs]

        def getCustomerReferralVouchers(customer):
            cvs = CustomerVoucher.objects.filter(**{
                'customer': customer,
                'voucher__category': getConstant('referrals__referrerCategory'),
            })
            return [cv.voucher for cv in cvs]

        Customer.add_to_class('getAvailableCredits',creditsAvailable)
        Customer.add_to_class('getVouchers',getCustomerVouchers)
        Customer.add_to_class('getReferralVouchers',getCustomerReferralVouchers)

        # This ensures that the signal receivers are loaded
        from . import handlers



from django.dispatch import receiver
from django.db.models import Q
from django.apps import apps

import logging
from collections import OrderedDict

from danceschool.core.signals import request_discounts, apply_discount, apply_addons, post_registration
from danceschool.core.constants import getConstant
from danceschool.core.models import Customer
from danceschool.core.classreg import RegistrationSummaryView

from .helpers import getApplicableDiscountCombos
from .models import DiscountCombo, TemporaryRegistrationDiscount, RegistrationDiscount


# Define logger for this file
logger = logging.getLogger(__name__)


@receiver(request_discounts)
def getBestDiscount(sender,**kwargs):
    '''
    When a customer registers for events, discounts may need to be
    automatically applied.  A given shopping cart may, in fact,
    be eligible for multiple different types of discounts (e.g. hours-based
    discounts for increasing numbers of class hours), but typically, only one
    discount should be applied.  Therefore, this handler loops through all potential
    discounts, finds the ones that are applicable to the passed registration or set
    of items, and returns the code and discounted price of the best available discount,
    in a tuple of the form (code, discounted_price).
    '''
    if not getConstant('general__discountsEnabled'):
        return

    logger.debug('Signal fired to request discounts.')

    reg = kwargs.pop('registration',None)
    if not reg:
        logger.warning('No registration passed, discounts not applied.')
        return

    payAtDoor = reg.payAtDoor

    # Check if this is a new customer, who may be eligible for special discounts
    newCustomer = True
    customer = Customer.objects.filter(email=reg.email,first_name=reg.firstName,last_name=reg.lastName).first()
    if (customer and customer.numClassSeries > 0) or sender != RegistrationSummaryView:
        newCustomer = False

    eligible_filter = (
        Q(event__series__pricingTier__isnull=False) |
        Q(event__publicevent__pricingTier__isnull=False)
    )
    ineligible_filter = (
        (Q(event__series__isnull=False) & Q(event__series__pricingTier__isnull=True)) |
        (Q(event__publicevent__isnull=False) & Q(event__publicevent__pricingTier__isnull=True)) |
        Q(dropIn=True)
    )
    if apps.is_installed('danceschool.private_lessons'):
        eligible_filter = eligible_filter | Q(event__privatelessonevent__pricingTier__isnull=False)
        ineligible_filter = ineligible_filter | (
            Q(event__privatelessonevent__isnull=False) &
            Q(event__privatelessonevent__pricingTier__isnull=True)
        )

    # The items for which the customer registered.
    eventregs_list = reg.temporaryeventregistration_set.all()
    eligible_list = eventregs_list.filter(dropIn=False).filter(eligible_filter)
    ineligible_list = eventregs_list.filter(ineligible_filter)

    ineligible_total = sum(
        [x.event.getBasePrice(payAtDoor=payAtDoor) for x in ineligible_list.exclude(dropIn=True)] +
        [x.price for x in ineligible_list.filter(dropIn=True)]
    )

    # Get the applicable discounts and sort them in ascending category order
    # so that the best discounts are always listed in the order that they will
    # be applied.
    discountCodesApplicable = getApplicableDiscountCombos(eligible_list, newCustomer, reg.student, customer=customer, addOn=False, cannotCombine=False)
    discountCodesApplicable.sort(key=lambda x: x.code.category.order)

    # Once we have a list of codes to try, calculate the discounted price for each possibility,
    # and pick the one in each category that has the lowest total price.  We also need to keep track
    # of the way in which some discounts are allocated across individual events.
    best_discounts = OrderedDict()

    initial_prices = [x.event.getBasePrice(payAtDoor=payAtDoor) for x in eligible_list]
    initial_total = sum(initial_prices)

    if discountCodesApplicable:
        net_allocated_prices = initial_prices
        net_precategory_price = initial_total
        last_category = discountCodesApplicable[0].code.category

    for discount in discountCodesApplicable:

        # If the category has changed, then the new net_allocated_prices and the
        # new net_precategory price are whatever was found to be best in the last category.
        if (discount.code.category != last_category):
            last_category = discount.code.category

            if best_discounts:
                # Since this is an OrderedDict, we can get the last element of the dict from
                # the iterator, which is the last category for which there was a valid discount.
                last_discount = best_discounts.get(next(reversed(best_discounts)))
                net_allocated_prices = last_discount.net_allocated_prices
                net_precategory_price = last_discount.net_price

        # The second item in each tuple is now adjusted, so that each item that is wholly or partially
        # applied against the discount will be wholly (value goes to 0) or partially subtracted from the
        # remaining value to be calculated at full price.
        tieredTuples = [(x,1) for x in eligible_list[:]]

        for itemTuple in discount.itemTuples:
            tieredTuples = [(p,q) if p != itemTuple[0] else (p,q - itemTuple[1]) for (p,q) in tieredTuples]

        response = discount.code.applyAndAllocate(net_allocated_prices,tieredTuples,payAtDoor)

        # Once the final price has been calculated, apply it iff it is less than
        # the previously best discount found.
        current_code = best_discounts.get(discount.code.category.name, None)
        if (
            response and (
                (not current_code and response.net_price < net_precategory_price) or
                (current_code and response.net_price < current_code.net_price)
            )
        ):
            best_discounts[discount.code.category.name] = response

    # Now, repeat the basic process for codes that cannot be combined.  These codes are always
    # compared against the base price, and there is no need to allocate across items since
    # only one code will potentially be applied.
    uncombinedCodesApplicable = getApplicableDiscountCombos(
        eligible_list, newCustomer, reg.student, customer=customer, addOn=False, cannotCombine=True
    )

    for discount in uncombinedCodesApplicable:

        # The second item in each tuple is now adjusted, so that each item that is wholly or partially
        # applied against the discount will be wholly (value goes to 0) or partially subtracted from the
        # remaining value to be calculated at full price.
        tieredTuples = [(x,1) for x in eligible_list[:]]

        for itemTuple in discount.itemTuples:
            tieredTuples = [(p,q) if p != itemTuple[0] else (p,q - itemTuple[1]) for (p,q) in tieredTuples]

        response = discount.code.applyAndAllocate(initial_prices,tieredTuples,payAtDoor)

        # Once the final price has been calculated, apply it iff it is less than
        # the previously best discount or combination of discounts found.
        if (
            response and
            response.net_price < min([x.net_price for x in best_discounts.values()] + [initial_total])
        ):
            best_discounts = OrderedDict({discount.code.category.name: response})

    if not best_discounts:
        logger.debug('No applicable discounts found.')

    # Return the list of discounts to be applied (in DiscountInfo tuples), along with the additional
    # price of ineligible items to be added.
    return DiscountCombo.DiscountApplication([x for x in best_discounts.values()], ineligible_total)


@receiver(apply_discount)
def applyTemporaryDiscount(sender,**kwargs):
    # Get the registration and the customer
    if not getConstant('general__discountsEnabled'):
        return

    logger.debug('Signal fired to apply discounts.')

    reg = kwargs.pop('registration',None)
    discount = kwargs.pop('discount',None)
    discountAmount = kwargs.pop('discount_amount',None)

    if not reg or not discount:
        logger.warning('Incomplete information passed, discounts not applied.')
        return

    obj, created = TemporaryRegistrationDiscount.objects.update_or_create(
        registration=reg,
        discount=discount,
        defaults={'discountAmount': discountAmount,},
    )
    logger.debug('Discount applied.')
    return obj


@receiver(apply_addons)
def getAddonItems(sender, **kwargs):
    # Check if this is a new customer
    if not getConstant('general__discountsEnabled'):
        return

    logger.debug('Signal fired to request free add-ons.')

    reg = kwargs.pop('registration',None)
    if not reg:
        logger.warning('No registration passed, addons not applied.')
        return

    newCustomer = True
    customer = Customer.objects.filter(email=reg.email,first_name=reg.firstName,last_name=reg.lastName).first()
    if (customer and customer.numClassSeries > 0) or sender != RegistrationSummaryView:
        newCustomer = False

    # No need to get all objects, just the ones that could qualify one for an add-on
    cart_object_list = reg.temporaryeventregistration_set.filter(dropIn=False).filter(Q(event__series__pricingTier__isnull=False) | Q(event__publicevent__pricingTier__isnull=False))

    availableAddons = getApplicableDiscountCombos(cart_object_list, newCustomer, reg.student, customer=customer, addOn=True)
    return [x.code.name for x in availableAddons]


@receiver(post_registration)
def applyFinalDiscount(sender,**kwargs):
    # Get the registration and the customer
    if not getConstant('general__discountsEnabled'):
        return

    logger.debug('Signal fired to finalize application of discounts.')

    reg = kwargs.pop('registration',None)

    if not reg:
        logger.debug('No registration passed, discounts not applied.')
        return

    trds = TemporaryRegistrationDiscount.objects.filter(registration=reg.temporaryRegistration)

    obj = None
    for temp_discount in trds:
        obj = RegistrationDiscount.objects.create(
            registration=reg,
            discount=temp_discount.discount,
            discountAmount=temp_discount.discountAmount,
        )

    logger.debug('Discounts applied.')
    return obj



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-08-29 17:01
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0014_auto_20170821_1226'),
    ]

    operations = [
        migrations.AddField(
            model_name='temporaryregistration',
            name='expirationDate',
            field=models.DateTimeField(default=django.utils.timezone.now, help_text='When a customer attempts to begin the registration process, the system looks for temporary registrations that are still in progress (with a future expiration date) to determine if there is space for them to register.', verbose_name='Expiration date'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='temporaryregistration',
            name='email',
            field=models.CharField(max_length=200, null=True, verbose_name='Email address'),
        ),
        migrations.AlterField(
            model_name='temporaryregistration',
            name='firstName',
            field=models.CharField(max_length=100, null=True, verbose_name='First name'),
        ),
        migrations.AlterField(
            model_name='temporaryregistration',
            name='lastName',
            field=models.CharField(max_length=100, null=True, verbose_name='Last name'),
        ),
    ]



from django.conf.urls import url

from .feeds import json_availability_feed, json_lesson_feed
from .views import BookPrivateLessonView, PrivateLessonStudentInfoView, InstructorAvailabilityView, AddAvailabilitySlotView, UpdateAvailabilitySlotView

urlpatterns = [
    # JSON private lesson availability feeds
    url(r'^availability/feed/json/(?P<instructor_id>[\w\-_]+)/$', json_availability_feed, name='jsonPrivateLessonAvailabilityFeed'),
    url(r'^availability/feed/json/$', json_availability_feed, name='jsonPrivateLessonAvailabilityFeed'),

    # JSON scheduled lesson feeds
    url(r'^scheduled/feed/json/$', json_lesson_feed, name='jsonOwnPrivateLessonFeed'),
    url(r'^scheduled/feed/json/all/$', json_lesson_feed, {'show_others': True}, name='jsonPrivateLessonFeed'),
    url(r'^scheduled/feed/json/(?P<location_id>[0-9]+)/(?P<room_id>[0-9]+)/$', json_lesson_feed, name='jsonOwnPrivateLessonFeed'),
    url(r'^scheduled/feed/json/(?P<location_id>[0-9]+)/(?P<room_id>[0-9]+)/all/$', json_lesson_feed, {'show_others': True}, name='jsonPrivateLessonFeed'),
    url(r'^scheduled/feed/json/(?P<location_id>[0-9]+)/$', json_lesson_feed, name='jsonOwnPrivateLessonFeed'),
    url(r'^scheduled/feed/json/(?P<location_id>[0-9]+)/all/$', json_lesson_feed, {'show_others': True}, name='jsonPrivateLessonFeed'),

    # Views for scheduling a private lesson
    url(r'^schedule/$', BookPrivateLessonView.as_view(), name='bookPrivateLesson'),
    url(r'^schedule/getinfo/$', PrivateLessonStudentInfoView.as_view(), name='privateLessonStudentInfo'),

    # Views for setting instructor availability
    url(r'^instructor/availability/$', InstructorAvailabilityView.as_view(), name='instructorAvailability'),
    url(r'^instructor/availability/add/$', AddAvailabilitySlotView.as_view(), name='addAvailabilitySlot'),
    url(r'^instructor/availability/update/$', UpdateAvailabilitySlotView.as_view(), name='updateAvailabilitySlot'),
]



from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django.forms import ModelForm

from danceschool.core.admin import EventChildAdmin, EventOccurrenceInline
from danceschool.core.models import Event
from danceschool.core.forms import LocationWithDataWidget

from .models import PrivateEvent, PrivateEventCategory, EventReminder


class EventReminderInline(admin.StackedInline):
    model = EventReminder
    extra = 0


class PrivateEventAdminForm(ModelForm):
    '''
    Custom form for private events is needed to include necessary
    Javascript for room selection, even though capacity is not
    an included field in this admin.
    '''

    class Meta:
        model = PrivateEvent
        exclude = ['month','year','startTime','endTime','duration','submissionUser','registrationOpen','capacity','status']
        widgets = {
            'location': LocationWithDataWidget,
        }

    class Media:
        js = ('js/serieslocation_capacity_change.js','js/location_related_objects_lookup.js')


class PrivateEventAdmin(EventChildAdmin):
    base_model = PrivateEvent
    form = PrivateEventAdminForm
    show_in_index = True

    list_display = ('name','category','nextOccurrenceTime','firstOccurrenceTime','location_given','displayToGroup')
    list_filter = ('category','displayToGroup','location','locationString')
    search_fields = ('title',)
    ordering = ('-endTime',)
    inlines = [EventOccurrenceInline, EventReminderInline]

    fieldsets = (
        (None, {
            'fields': ('title','category','descriptionField','link')
        }),
        ('Location', {
            'fields': (('location','room'),'locationString')
        }),
        ('Visibility', {
            'fields': ('displayToGroup','displayToUsers'),
        })
    )

    def location_given(self,obj):
        if obj.room and obj.location:
            return _('%s, %s' % (obj.room.name, obj.location.name))
        if obj.location:
            return obj.location.name
        return obj.locationString

    def save_model(self,request,obj,form,change):
        obj.status = Event.RegStatus.disabled
        obj.submissionUser = request.user
        obj.save()


admin.site.register(PrivateEvent,PrivateEventAdmin)
admin.site.register(PrivateEventCategory)



from django.http import JsonResponse
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.utils import timezone

from django_ical.views import ICalFeed
from datetime import datetime, timedelta
import pytz

from .models import EventOccurrence, StaffMember, Event
from .constants import getConstant
from .utils.timezone import ensure_timezone


# Because our calendar will have both series classes and non-recurring events
# in it, we need to create a custom class with properties assigned appropriately
# to iterate through in the feed generation process
class EventFeedItem(object):

    def __init__(self,object,**kwargs):

        timeZone = pytz.timezone(getattr(settings,'TIME_ZONE','UTC'))
        if kwargs.get('timeZone',None):
            try:
                timeZone = pytz.timezone(kwargs.get('timeZone',None))
            except pytz.exceptions.UnknownTimeZoneError:
                pass

        self.id = 'event_' + str(object.event.id) + '_' + str(object.id)
        self.type = 'event'
        self.id_number = object.event.id
        self.title = object.event.name
        self.description = object.event.description
        self.start = timezone.localtime(object.startTime,timeZone) \
            if timezone.is_aware(object.startTime) else object.startTime
        self.end = timezone.localtime(object.endTime,timeZone) \
            if timezone.is_aware(object.endTime) else object.endTime
        self.color = object.event.displayColor
        self.url = object.event.get_absolute_url()
        if hasattr(object,'event.location'):
            self.location = object.event.location.name + '\n' + object.event.location.address + '\n' + object.event.location.city + ', ' + object.event.location.state + ' ' + object.event.location.zip
        else:
            self.location = None


class EventFeed(ICalFeed):
    """
    A simple event calender
    """
    timezone = getattr(settings,'TIME_ZONE','UTC')

    def get_object(self,request,instructorFeedKey=''):
        if instructorFeedKey:
            return instructorFeedKey
        else:
            return None

    def title(self,obj):
        businessName = getConstant('contact__businessName')

        if not obj:
            return _('%s Events Calendar' % businessName)
        else:
            this_instructor = StaffMember.objects.filter(**{'feedKey': obj}).values('firstName','lastName').first()
            return _('%s Instructor Calendar for %s %s' % (businessName,this_instructor['firstName'],this_instructor['lastName']))

    def description(self):
        return _('Calendar for %s' % getConstant('contact__businessName'))

    def items(self,obj):
        if not getConstant('calendar__calendarFeedEnabled'):
            return []

        item_set = EventOccurrence.objects.exclude(event__status=Event.RegStatus.hidden).filter(Q(event__series__isnull=False) | Q(event__publicevent__isnull=False)).order_by('-startTime')

        if not obj:
            # Public calendar does not show hidden Events _or_ link-only registration Events
            return [EventFeedItem(x) for x in item_set.exclude(event__status=Event.RegStatus.linkOnly)[:100]]
        else:
            # Private calendars do show link-only registration Events
            return [EventFeedItem(x) for x in item_set.filter(event__eventstaffmember__staffMember__feedKey=obj)[:100]]

    def item_guid(self,item):
        return item.id + '@' + item.url

    def item_title(self, item):
        return item.title

    def item_link(self, item):
        return item.url

    def item_location(self, item):
        return item.location

    def item_description(self, item):
        return item.description

    def item_start_datetime(self, item):
        return item.start

    def item_end_datetime(self, item):
        return item.end


# The Jquery fullcalendar app requires a JSON news feed, so this function
# creates the feed from upcoming SeriesClass and Event objects.
def json_event_feed(request,instructorFeedKey='',locationId=None,roomId=None):

    if not getConstant('calendar__calendarFeedEnabled'):
        return JsonResponse({})

    startDate = request.GET.get('start','')
    endDate = request.GET.get('end','')
    timeZone = request.GET.get('timezone',getattr(settings,'TIME_ZONE','UTC'))

    filters = Q(event__month__isnull=False) & Q(event__year__isnull=False) & (Q(event__series__isnull=False) | Q(event__publicevent__isnull=False))
    exclusions = Q(event__status=Event.RegStatus.hidden)
    if startDate:
        limit_time = ensure_timezone(datetime.strptime(startDate,'%Y-%m-%d'))
        filters = filters & Q(startTime__gte=limit_time)
    if endDate:
        limit_time = ensure_timezone(datetime.strptime(endDate,'%Y-%m-%d'))
        filters = filters & Q(endTime__lte=limit_time + timedelta(days=1))

    if locationId:
        filters = filters & Q(event__location__id=locationId)
    if roomId:
        filters = filters & Q(event__room_id=roomId)

    if instructorFeedKey:
        # Private calendars do show link-only registration Events
        filters = filters & Q(event__eventstaffmember__staffMember__feedKey=instructorFeedKey)
    else:
        # Public calendar does not show hidden Events _or_ link-only registration Events
        exclusions = exclusions | Q(event__status=Event.RegStatus.linkOnly)

    item_set = EventOccurrence.objects.exclude(exclusions).filter(filters).order_by('-startTime')
    eventlist = [EventFeedItem(x,timeZone=timeZone).__dict__ for x in item_set]
    return JsonResponse(eventlist,safe=False)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-08-30 15:59
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0015_auto_20170829_1301'),
    ]

    operations = [
        migrations.RenameField(
            model_name='pricingtier',
            old_name='doorGeneralPrice',
            new_name='doorPrice',
        ),
        migrations.RenameField(
            model_name='pricingtier',
            old_name='onlineGeneralPrice',
            new_name='onlinePrice',
        ),
        migrations.RemoveField(
            model_name='pricingtier',
            name='doorStudentPrice',
        ),
        migrations.RemoveField(
            model_name='pricingtier',
            name='onlineStudentPrice',
        ),
        migrations.AlterField(
            model_name='invoice',
            name='temporaryRegistration',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.TemporaryRegistration', verbose_name='Temporary registration'),
        ),
        migrations.AlterField(
            model_name='invoiceitem',
            name='temporaryEventRegistration',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.TemporaryEventRegistration', verbose_name='Temporary event registration'),
        ),
        migrations.AlterField(
            model_name='registration',
            name='temporaryRegistration',
            field=models.OneToOneField(null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.TemporaryRegistration', verbose_name='Associated temporary registration'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-11 03:52
from __future__ import unicode_literals

from django.db import migrations, models
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0017_auto_20170909_0024'),
    ]

    operations = [
        migrations.AlterField(
            model_name='invoice',
            name='data',
            field=jsonfield.fields.JSONField(blank=True, default={}, verbose_name='Additional data'),
        ),
        migrations.AddField(
            model_name='invoice',
            name='email',
            field=models.CharField(blank=True, max_length=200, null=True, verbose_name='Recipient email address'),
        ),
        migrations.AddField(
            model_name='invoice',
            name='firstName',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Recipient first name'),
        ),
        migrations.AddField(
            model_name='invoice',
            name='lastName',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Recipient last name'),
        ),
        migrations.AlterField(
            model_name='customer',
            name='data',
            field=jsonfield.fields.JSONField(blank=True, default={}, verbose_name='Additional data'),
        ),
        migrations.AlterField(
            model_name='eventregistration',
            name='data',
            field=jsonfield.fields.JSONField(blank=True, default={}, verbose_name='Additional data'),
        ),
        migrations.AlterField(
            model_name='registration',
            name='data',
            field=jsonfield.fields.JSONField(blank=True, default={}, verbose_name='Additional data'),
        ),
        migrations.AlterField(
            model_name='temporaryeventregistration',
            name='data',
            field=jsonfield.fields.JSONField(blank=True, default={}, verbose_name='Additional data'),
        ),
        migrations.AlterField(
            model_name='temporaryregistration',
            name='data',
            field=jsonfield.fields.JSONField(blank=True, default={}, verbose_name='Additional data'),
        ),
    ]



from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.apps import apps

from cms.toolbar_pool import toolbar_pool
from cms.toolbar_base import CMSToolbar
from cms.toolbar.items import Break, SubMenu
from cms.cms_toolbars import ADMIN_MENU_IDENTIFIER, ADMINISTRATION_BREAK
from cms.constants import RIGHT


@toolbar_pool.register
class SettingsToolbar(CMSToolbar):
    ''' Adds a link to the Global settings page '''

    def populate(self):
        #
        # 'Apps' is the spot on the existing djang-cms toolbar admin_menu
        # 'where we'll insert all of our applications' menus.
        #
        if not self.request.user.has_perm('dynamic_preferences.change_global_preference'):
            return

        admin_menu = self.toolbar.get_or_create_menu(
            ADMIN_MENU_IDENTIFIER, _('Apps'))

        position = admin_menu.get_alphabetical_insert_position(
            _('Global Settings'),
            SubMenu
        )

        #
        # If zero was returned, then we know we're the first of our
        # applications' menus to be inserted into the admin_menu, so, here
        # we'll compute that we need to go after the first
        # ADMINISTRATION_BREAK and, we'll insert our own break after our
        # section.
        #
        if not position:
            # OK, use the ADMINISTRATION_BREAK location + 1
            position = admin_menu.find_first(
                Break,
                identifier=ADMINISTRATION_BREAK
            ) + 1
            # Insert our own menu-break, at this new position. We'll insert
            # all subsequent menus before this, so it will ultimately come
            # after all of our applications' menus.
            admin_menu.add_break('custom-break', position=position)

        # OK, create the global settings menu here
        admin_menu.add_link_item(
            _('Global Settings'), url=reverse('dynamic_preferences.global'),
            position=position
        )


@toolbar_pool.register
class EventsToolbar(CMSToolbar):
    ''' Adds items to the toolbar to add class Series and Events, etc, as well as a button to view registrations '''

    def populate(self):
        menu = self.toolbar.get_or_create_menu('core-events', _('Events'))

        if self.request.user.has_perm('core.view_registration_summary'):
            self.toolbar.add_button(_('View Registrations'), url=reverse('viewregistrations_selectevent'),side=RIGHT)

            menu.add_link_item(_('View Registrations'), url=reverse('viewregistrations_selectevent'))

        menu.add_link_item(_('Registration Page'), url=reverse('registration'))

        if (
            self.request.user.has_perm('core.add_series') or
            self.request.user.has_perm('core.add_publicevent') or
            self.request.user.has_perm('core.change_series') or
                self.request.user.has_perm('core.change_publicevent')):
                menu.add_break('post_registration_break')
                if self.request.user.has_perm('core.add_series'):
                    menu.add_link_item(_('Add a Class Series'), url=reverse('admin:core_series_add'))
                if self.request.user.has_perm('core.add_publicevent'):
                    menu.add_link_item(_('Add a Public Event'), url=reverse('admin:core_publicevent_add'))
                if self.request.user.has_perm('core.add_series'):
                    menu.add_link_item(_('Edit Existing Series'), url=reverse('admin:core_series_changelist'))
                if self.request.user.has_perm('core.add_publicevent'):
                    menu.add_link_item(_('Edit Existing Events'), url=reverse('admin:core_publicevent_changelist'))

        if self.request.user.has_perm('core.change_classdescription') or self.request.user.has_perm('core.change_dancetypelevel') \
            or self.request.user.has_perm('core.change_location') or self.request.user.has_perm('core.change_pricingtier') or \
                self.request.user.has_perm('core.change_customer') or self.request.user.has_perm('core.change_dancerole') or \
                self.request.user.has_perm('core.change_publiceventcategory') or self.request.user.has_perm('core.change_eventstaffcategory'):
                    menu.add_break('related_items_break')

        if self.request.user.has_perm('core.change_classdescription'):
            related_menu = menu.get_or_create_menu('core-events-related',_('Related Items'))
            related_menu.add_link_item(_('Class Descriptions'),url=reverse('admin:core_classdescription_changelist'))

        if self.request.user.has_perm('core.change_customer'):
            related_menu = menu.get_or_create_menu('core-events-related',_('Related Items'))
            related_menu.add_link_item(_('Customers'),url=reverse('admin:core_customer_changelist'))

        if self.request.user.has_perm('core.change_dancetypelevel'):
            related_menu = menu.get_or_create_menu('core-events-related',_('Related Items'))
            related_menu.add_link_item(_('Dance Roles'),url=reverse('admin:core_dancerole_changelist'))

        if self.request.user.has_perm('core.change_dancetypelevel'):
            related_menu = menu.get_or_create_menu('core-events-related',_('Related Items'))
            related_menu.add_link_item(_('Dance Types/Levels'),url=reverse('admin:core_dancetypelevel_changelist'))

        if self.request.user.has_perm('core.change_eventstaffcategory'):
            related_menu = menu.get_or_create_menu('core-events-related',_('Related Items'))
            related_menu.add_link_item(_('Event Staff Categories'),url=reverse('admin:core_eventstaffcategory_changelist'))

        if self.request.user.has_perm('core.change_location'):
            related_menu = menu.get_or_create_menu('core-events-related',_('Related Items'))
            related_menu.add_link_item(_('Locations'),url=reverse('admin:core_location_changelist'))

        if self.request.user.has_perm('core.change_pricingtier'):
            related_menu = menu.get_or_create_menu('core-events-related',_('Related Items'))
            related_menu.add_link_item(_('Pricing Tiers'),url=reverse('admin:core_pricingtier_changelist'))

        if self.request.user.has_perm('core.change_publiceventcategory'):
            related_menu = menu.get_or_create_menu('core-events-related',_('Related Items'))
            related_menu.add_link_item(_('Public Event Categories'),url=reverse('admin:core_publiceventcategory_changelist'))


@toolbar_pool.register
class StaffMemberToolbar(CMSToolbar):
    ''' Adds items to the toolbar to add class Series and Events. '''

    def populate(self):
        if hasattr(self.request.user,'staffmember') and hasattr(self.request.user.staffmember,'instructor') and self.request.user.has_perm('core.view_own_instructor_stats'):
            menu = self.toolbar.get_or_create_menu('core-staffmember', _('Staff'))
            menu.add_link_item(_('Your Stats'), url=reverse('instructorStats'))
        if hasattr(self.request.user,'staffmember') and self.request.user.has_perm('core.update_instructor_bio'):
            menu = self.toolbar.get_or_create_menu('core-staffmember', _('Staff'))
            menu.add_link_item(_('Update Your Contact Info'), url=reverse('instructorBioChange'))

        if self.request.user.has_perm('core.view_staff_directory'):
            menu = self.toolbar.get_or_create_menu('core-staffmember', _('Staff'))
            menu.add_link_item(_('Instructor/Staff Directory'), url=reverse('staffDirectory'))
            menu.add_break('post_directory_break')

        addBreak = False

        if self.request.user.has_perm('core.send_email'):
            menu = self.toolbar.get_or_create_menu('core-staffmember', _('Staff'))
            menu.add_link_item(_('Email Students'), url=reverse('emailStudents'))
            addBreak = True

        if self.request.user.has_perm('core.report_substitute_teaching'):
            menu = self.toolbar.get_or_create_menu('core-staffmember', _('Staff'))
            menu.add_link_item(_('Report Substitute Teaching'), url=reverse('substituteTeacherForm'))
            addBreak = True

        if addBreak:
            menu.add_break('post_instructor_functions_break')

        if self.request.user.has_perm('core.change_instructor'):
            menu = self.toolbar.get_or_create_menu('core-staffmember', _('Staff'))
            menu.add_link_item(_('Manage Instructors'), url=reverse('admin:core_instructor_changelist'))


@toolbar_pool.register
class ContentToolbar(CMSToolbar):
    ''' Adds links to manage files, and can be hooked in to manage News, FAQs, Surveys/Forms, etc. '''

    def populate(self):
        if (
            self.request.user.has_perm('filer.change_folder') or self.request.user.has_perm('core.change_emailtemplate') or
            (apps.is_installed('djangocms_forms') and self.request.user.has_perm('djangocms_forms.export_formsubmission'))
        ):
            menu = self.toolbar.get_or_create_menu('core-content',_('Content'))

        if self.request.user.has_perm('filer.change_folder'):
            menu.add_link_item(_('Manage Uploaded Files'), reverse('admin:filer_folder_changelist'))

        if self.request.user.has_perm('core.change_emailtemplate'):
            menu.add_link_item(_('Manage Email Templates'), reverse('admin:core_emailtemplate_changelist'))

        if apps.is_installed('djangocms_forms') and self.request.user.has_perm('djangocms_forms.export_formsubmission'):
            menu.add_link_item(_('View/Export Survey Responses'), reverse('admin:djangocms_forms_formsubmission_changelist'))


@toolbar_pool.register
class CoreFinancesToolbar(CMSToolbar):
    ''' Adds links to view and add invoices '''

    def populate(self):
        if (
            self.request.user.has_perm('core.add_invoice')
        ):
            menu = self.toolbar.get_or_create_menu('financial', _('Finances'))

            # The "Create an Invoice" link goes at the top of the menu
            menu.add_link_item(_('Create an Invoice'), url=reverse('admin:core_invoice_add'),position=0)

            # Other apps may have added related items already
            startPosition = menu.find_first(
                Break,
                identifier='financial_related_items_break'
            )
            if not startPosition:
                menu.add_break('financial_related_items_break')
                startPosition = menu.find_first(
                    Break,
                    identifier='financial_related_items_break'
                )
            related_menu = menu.get_or_create_menu('financial-related',_('Related Items'), position=startPosition + 2)
            related_menu.add_link_item(_('Invoices'), url=reverse('admin:core_invoice_changelist'))



# Give this app a custom verbose name to avoid confusion
from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class StatsAppConfig(AppConfig):
    name = 'danceschool.stats'
    verbose_name = _('Stats Functions')



from django.http import HttpResponse, JsonResponse
from django.utils.translation import ugettext_lazy as _
from django.views.generic import View
from django.contrib import messages
from django.core.exceptions import ObjectDoesNotExist
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import ensure_csrf_cookie

from .models import EventOccurrence, SeriesTeacher, EventRegistration, EmailTemplate


class UserAccountInfo(View):
    ''' This view just returns the name and email address information for the currently logged in user '''

    @method_decorator(ensure_csrf_cookie)
    def dispatch(self, request, *args, **kwargs):
        context = {}

        if not request.user.is_authenticated:
            return JsonResponse(context)

        customer = getattr(request.user,'customer',None)

        if customer:
            context.update({
                'customer': True,
                'first_name': customer.first_name or request.user.first_name,
                'last_name': customer.last_name or request.user.last_name,
                'email': customer.email or request.user.email,
                'phone': customer.phone,
            })
        else:
            context.update({
                'customer': False,
                'first_name': request.user.first_name,
                'last_name': request.user.last_name,
                'email': request.user.email,
            })

        # Also add any outstanding messages (e.g. login successful message) to be
        # relayed to the user when this information is used.
        context['messages'] = []

        for message in messages.get_messages(request):
            context['messages'].append({
                "level": message.level,
                "message": message.message,
                "extra_tags": message.tags,
            })

        return JsonResponse(context)


def updateSeriesAttributes(request):
    '''
    This function handles the filtering of available series classes and seriesteachers when a series
    is chosen on the Substitute Teacher reporting form.
    '''
    if request.method == 'POST' and request.POST.get('event'):
        series_option = request.POST.get('event') or None
        seriesClasses = EventOccurrence.objects.filter(event__id=series_option)
        seriesTeachers = SeriesTeacher.objects.filter(event__id=series_option)
    else:
        # Only return attributes for valid requests
        return JsonResponse({})

    outClasses = {}
    for option in seriesClasses:
        outClasses[str(option.id)] = option.__str__()

    outTeachers = {}
    for option in seriesTeachers:
        outTeachers[str(option.id)] = option.__str__()

    return JsonResponse({
        'id_occurrences': outClasses,
        'id_replacedStaffMember': outTeachers,
    })


def processCheckIn(request):
    '''
    This function handles the Ajax call made when a user is marked as checked in
    '''

    if request.method == 'POST':
        event_id = request.POST.get('event_id')
        reg_ids = request.POST.getlist('reg_id')

        if not event_id:
            return HttpResponse(_("Error at start."))

        # Get all possible registrations, so that we can set those that are not included to False (and set those that are included to True)
        all_eventreg = list(EventRegistration.objects.filter(event__id=event_id))

        for this_reg in all_eventreg:
            if str(this_reg.registration.id) in reg_ids and not this_reg.checkedIn:
                this_reg.checkedIn = True
                this_reg.save()
            elif str(this_reg.registration.id) not in reg_ids and this_reg.checkedIn:
                this_reg.checkedIn = False
                this_reg.save()
        return HttpResponse("OK.")


def getEmailTemplate(request):
    '''
    This function handles the Ajax call made when a user wants a specific email template
    '''
    if request.method != 'POST':
        return HttpResponse(_('Error, no POST data.'))

    if not hasattr(request,'user'):
        return HttpResponse(_('Error, not authenticated.'))

    template_id = request.POST.get('template')

    if not template_id:
        return HttpResponse(_("Error, no template ID provided."))

    try:
        this_template = EmailTemplate.objects.get(id=template_id)
    except ObjectDoesNotExist:
        return HttpResponse(_("Error getting template."))

    if this_template.groupRequired and this_template.groupRequired not in request.user.groups.all():
        return HttpResponse(_("Error, no permission to access this template."))
    if this_template.hideFromForm:
        return HttpResponse(_("Error, no permission to access this template."))

    return JsonResponse({
        'subject': this_template.subject,
        'content': this_template.content,
        'html_content': this_template.html_content,
        'richTextChoice': this_template.richTextChoice,
    })



from django.template import Template, Context
from django.views.generic import FormView
from django.test import RequestFactory
from django.utils.translation import ugettext_lazy as _
from django.db import IntegrityError
from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpResponseBadRequest, HttpResponseRedirect
from django.core.urlresolvers import reverse

from easy_pdf.views import PDFTemplateView
import re
import logging

from danceschool.core.models import Invoice
from danceschool.core.constants import getConstant, INVOICE_VALIDATION_STR
from danceschool.core.mixins import EmailRecipientMixin
from danceschool.core.helpers import emailErrorMessage

from .forms import VoucherCustomizationForm
from .models import Voucher


# Define logger for this file
logger = logging.getLogger(__name__)


class GiftCertificateCustomizeView(FormView):
    template_name = 'cms/forms/display_form_classbased.html'
    form_class = VoucherCustomizationForm

    def dispatch(self,request,*args,**kwargs):
        '''
        Check that a valid Invoice ID has been passed in session data,
        and that said invoice is marked as paid.
        '''
        paymentSession = request.session.get(INVOICE_VALIDATION_STR, {})
        self.invoiceID = paymentSession.get('invoiceID')
        self.amount = paymentSession.get('amount',0)
        self.success_url = paymentSession.get('success_url',reverse('registration'))

        # Check that Invoice matching passed ID exists
        try:
            i = Invoice.objects.get(id=self.invoiceID)
        except ObjectDoesNotExist:
            return HttpResponseBadRequest(_('Invalid invoice information passed.'))

        if i.unpaid or i.amountPaid != self.amount:
            return HttpResponseBadRequest(_('Passed invoice is not paid.'))

        return super(GiftCertificateCustomizeView,self).dispatch(request,*args,**kwargs)

    def form_valid(self,form):
        '''
        Create the gift certificate voucher with the indicated information and send
        the email as directed.
        '''
        emailTo = form.cleaned_data.get('emailTo')
        emailType = form.cleaned_data.get('emailType')
        recipientName = form.cleaned_data.get('recipientName')
        fromName = form.cleaned_data.get('fromName')
        message = form.cleaned_data.get('message')

        logger.info('Processing gift certificate.')

        try:
            voucher = Voucher.create_new_code(
                prefix='GC_',
                name=_('Gift certificate: %s%s for %s' % (getConstant('general__currencySymbol'),self.amount, emailTo)),
                category=getConstant('vouchers__giftCertCategory'),
                originalAmount=self.amount,
                singleUse=False,
                forFirstTimeCustomersOnly=False,
                expirationDate=None,
            )
        except IntegrityError:
            logger.error('Error creating gift certificate voucher for Invoice #%s' % self.invoiceId)
            emailErrorMessage(_('Gift certificate transaction not completed'), self.invoiceId)

        template = getConstant('vouchers__giftCertTemplate')

        # Attempt to attach a PDF of the gift certificate
        rf = RequestFactory()
        pdf_request = rf.get('/')

        pdf_kwargs = {
            'currencySymbol': getConstant('general__currencySymbol'),
            'businessName': getConstant('contact__businessName'),
            'certificateAmount': voucher.originalAmount,
            'certificateCode': voucher.voucherId,
            'certificateMessage': message,
            'recipientName': recipientName,
            'fromName': fromName,
        }
        if recipientName:
            pdf_kwargs.update({})

        attachment = GiftCertificatePDFView(request=pdf_request).get(request=pdf_request,**pdf_kwargs).content or None

        if attachment:
            attachment_name = 'gift_certificate.pdf'
        else:
            attachment_name = None

        # Send a confirmation email
        email_class = EmailRecipientMixin()
        email_class.email_recipient(
            subject=template.subject,
            content=template.content,
            send_html=template.send_html,
            html_content=template.html_content,
            from_address=template.defaultFromAddress,
            from_name=template.defaultFromName,
            cc=template.defaultCC,
            to=emailTo,
            currencySymbol=getConstant('general__currencySymbol'),
            businessName=getConstant('contact__businessName'),
            certificateAmount=voucher.originalAmount,
            certificateCode=voucher.voucherId,
            certificateMessage=message,
            recipientName=recipientName,
            fromName=fromName,
            emailType=emailType,
            recipient_name=recipientName,
            attachment_name=attachment_name,
            attachment=attachment
        )

        # Remove the invoice session data
        self.request.session.pop(INVOICE_VALIDATION_STR,None)

        return HttpResponseRedirect(self.get_success_url())


class GiftCertificatePDFView(PDFTemplateView):
    template_name = 'vouchers/pdf/giftcertificate_template.html'

    def get_context_data(self,**kwargs):
        context = super(GiftCertificatePDFView,self).get_context_data(**kwargs)

        template = getConstant('vouchers__giftCertPDFTemplate')

        # For security reasons, the following tags are removed from the template before parsing:
        # {% extends %}{% load %}{% debug %}{% include %}{% ssi %}
        content = re.sub('\{%\s*((extends)|(load)|(debug)|(include)|(ssi))\s+.*?\s*%\}','',template.content)

        t = Template(content)

        rendered_content = t.render(Context(context))

        context.update({
            'header': template.subject,
            'content': rendered_content
        })

        return context



from django.contrib.auth.models import User
from django.utils.encoding import force_text
from django.db.models import Q

from dal import autocomplete

from .models import Customer


class UserAutoComplete(autocomplete.Select2QuerySetView):

    def get_queryset(self):
        # Filter out results for unauthenticated users.
        if not self.request.user.has_perm('core.can_autocomplete_users'):
            return User.objects.none()

        qs = User.objects.all()

        if self.q:
            words = self.q.split(' ')
            lastName = words.pop()
            firstName = words.pop() if words else lastName

            qs = qs.filter(
                Q(first_name__istartswith=firstName) | Q(last_name__istartswith=lastName) |
                Q(email__istartswith=self.q)
            )

        return qs

    def get_result_label(self,item):
        return force_text(item.get_full_name() + ': ' + item.email)


class CustomerAutoComplete(autocomplete.Select2QuerySetView):

    def get_queryset(self):
        # Filter out results for unauthenticated users.
        if not self.request.user.has_perm('core.can_autocomplete_users'):
            return Customer.objects.none()

        qs = Customer.objects.all()

        if self.q:
            words = self.q.split(' ')
            lastName = words.pop()
            firstName = words.pop() if words else lastName

            qs = qs.filter(
                Q(first_name__istartswith=firstName) | Q(last_name__istartswith=lastName) |
                Q(email__istartswith=self.q)
            )

        return qs
